"""Модуль IPFS кэша для GLT CLI."""

import hashlib
import json
import logging
import shutil
from pathlib import Path
from typing import Dict, Any, Optional, List
from urllib.parse import urlparse

import requests
from rich.console import Console

from glt_cli.models import GLTConfig

console = Console()
logger = logging.getLogger(__name__)


class IPFSCache:
    """IPFS кэш для шаблонов GLT."""
    
    def __init__(self, config: GLTConfig):
        """Инициализирует IPFS кэш.
        
        Args:
            config: GLT конфигурация
        """
        self.config = config
        self.cache_dir = Path.home() / ".glt" / "cache" / "ipfs"
        self.cache_dir.mkdir(parents=True, exist_ok=True)
        
        # Получаем IPFS endpoint из конфигурации
        self.ipfs_endpoint = None
        if config.infra and config.infra.cache_ipfs:
            self.ipfs_endpoint = config.infra.cache_ipfs
        
        # Проверяем alien_tech
        self.alien_tech = config.alien_tech
        
        logger.info(f"IPFS кэш инициализирован: {self.cache_dir}")
        logger.info(f"Alien tech: {self.alien_tech}")
    
    def get_template(self, template_hash: str) -> Optional[bytes]:
        """Получает шаблон из IPFS или кэша.
        
        Args:
            template_hash: IPFS хэш шаблона
            
        Returns:
            bytes: Содержимое шаблона или None
        """
        # Сначала проверяем локальный кэш
        cached_file = self.cache_dir / f"{template_hash}.template"
        if cached_file.exists():
            logger.info(f"Найден в кэше: {template_hash}")
            console.print(f"[dim]📦 Загружен из кэша: {template_hash}[/dim]")
            return cached_file.read_bytes()
        
        # Если нет в кэше, загружаем из IPFS
        if self.ipfs_endpoint:
            content = self._fetch_from_ipfs(template_hash)
            if content:
                # Сохраняем в кэш
                cached_file.write_bytes(content)
                console.print(f"[dim]📥 Загружен из IPFS: {template_hash}[/dim]")
                return content
        
        console.print(f"[yellow]⚠️  Шаблон не найден: {template_hash}[/yellow]")
        return None
    
    def cache_template(self, template_hash: str, content: bytes) -> bool:
        """Кэширует шаблон локально.
        
        Args:
            template_hash: IPFS хэш шаблона
            content: Содержимое шаблона
            
        Returns:
            bool: True если кэширование успешно
        """
        try:
            cached_file = self.cache_dir / f"{template_hash}.template"
            cached_file.write_bytes(content)
            
            # Создаем метаданные
            meta_file = self.cache_dir / f"{template_hash}.meta"
            metadata = {
                "hash": template_hash,
                "size": len(content),
                "cached_at": str(Path().stat().st_mtime),
                "alien_tech": self.alien_tech
            }
            meta_file.write_text(json.dumps(metadata, indent=2))
            
            logger.info(f"Шаблон закэширован: {template_hash}")
            return True
            
        except Exception as e:
            logger.error(f"Ошибка кэширования шаблона: {e}")
            return False
    
    def _fetch_from_ipfs(self, template_hash: str) -> Optional[bytes]:
        """Загружает шаблон из IPFS.
        
        Args:
            template_hash: IPFS хэш шаблона
            
        Returns:
            bytes: Содержимое шаблона или None
        """
        try:
            # Парсим endpoint
            parsed = urlparse(self.ipfs_endpoint)
            
            # Формируем URL для загрузки
            if parsed.scheme == "ipfs":
                # Прямой IPFS протокол
                url = f"https://ipfs.io/ipfs/{template_hash}"
            else:
                # HTTP endpoint
                url = f"{self.ipfs_endpoint}/ipfs/{template_hash}"
            
            logger.info(f"Загружаем из IPFS: {url}")
            
            response = requests.get(url, timeout=30)
            if response.status_code == 200:
                return response.content
            else:
                logger.error(f"Ошибка загрузки из IPFS: {response.status_code}")
                return None
                
        except requests.RequestException as e:
            logger.error(f"Ошибка запроса к IPFS: {e}")
            return None
        except Exception as e:
            logger.error(f"Неожиданная ошибка IPFS: {e}")
            return None
    
    def list_cached_templates(self) -> List[Dict[str, Any]]:
        """Список закэшированных шаблонов.
        
        Returns:
            List[Dict]: Список метаданных шаблонов
        """
        templates = []
        
        for meta_file in self.cache_dir.glob("*.meta"):
            try:
                metadata = json.loads(meta_file.read_text())
                templates.append(metadata)
            except Exception as e:
                logger.error(f"Ошибка чтения метаданных {meta_file}: {e}")
        
        return templates
    
    def clear_cache(self) -> bool:
        """Очищает кэш.
        
        Returns:
            bool: True если очистка успешна
        """
        try:
            if self.cache_dir.exists():
                shutil.rmtree(self.cache_dir)
                self.cache_dir.mkdir(parents=True)
            
            console.print("[green]✅ Кэш очищен[/green]")
            return True
            
        except Exception as e:
            console.print(f"[red]❌ Ошибка очистки кэша: {e}[/red]")
            return False
    
    def get_cache_stats(self) -> Dict[str, Any]:
        """Получает статистику кэша.
        
        Returns:
            Dict: Статистика кэша
        """
        try:
            template_files = list(self.cache_dir.glob("*.template"))
            total_size = sum(f.stat().st_size for f in template_files)
            
            return {
                "templates_count": len(template_files),
                "total_size_bytes": total_size,
                "total_size_mb": round(total_size / 1024 / 1024, 2),
                "cache_dir": str(self.cache_dir),
                "alien_tech": self.alien_tech,
                "ipfs_endpoint": self.ipfs_endpoint
            }
            
        except Exception as e:
            logger.error(f"Ошибка получения статистики кэша: {e}")
            return {}


class TemplateManager:
    """Менеджер шаблонов с IPFS поддержкой."""
    
    def __init__(self, config: GLTConfig):
        """Инициализирует менеджер шаблонов.
        
        Args:
            config: GLT конфигурация
        """
        self.config = config
        self.ipfs_cache = IPFSCache(config)
        self.templates_dir = Path("templates")
    
    def get_template(self, template_name: str) -> Optional[bytes]:
        """Получает шаблон по имени.
        
        Args:
            template_name: Имя шаблона
            
        Returns:
            bytes: Содержимое шаблона или None
        """
        # Сначала ищем локально
        local_template = self.templates_dir / template_name
        if local_template.exists():
            logger.info(f"Найден локальный шаблон: {template_name}")
            return local_template.read_bytes()
        
        # Если нет локально, ищем в IPFS кэше
        # Генерируем хэш из имени шаблона (упрощенная логика)
        template_hash = hashlib.sha256(template_name.encode()).hexdigest()
        return self.ipfs_cache.get_template(template_hash)
    
    def cache_template(self, template_name: str, content: bytes) -> bool:
        """Кэширует шаблон.
        
        Args:
            template_name: Имя шаблона
            content: Содержимое шаблона
            
        Returns:
            bool: True если кэширование успешно
        """
        template_hash = hashlib.sha256(template_name.encode()).hexdigest()
        return self.ipfs_cache.cache_template(template_hash, content)
    
    def setup_templates(self, project_dir: Path) -> bool:
        """Настраивает шаблоны для проекта.
        
        Args:
            project_dir: Директория проекта
            
        Returns:
            bool: True если настройка успешна
        """
        try:
            # Создаем директорию шаблонов
            templates_dir = project_dir / "templates"
            templates_dir.mkdir(exist_ok=True)
            
            # Создаем индекс шаблонов если есть alien_tech
            if self.config.alien_tech:
                index_file = templates_dir / "index.json"
                index_content = {
                    "alien_tech": self.config.alien_tech,
                    "templates": {
                        "docker": "QmDockerTemplateHash",
                        "ci": "QmCITemplateHash", 
                        "python": "QmPythonTemplateHash"
                    },
                    "cache_info": {
                        "endpoint": self.config.infra.cache_ipfs if self.config.infra else None,
                        "enabled": True
                    }
                }
                
                index_file.write_text(json.dumps(index_content, indent=2))
                console.print(f"[green]✅ Индекс шаблонов создан: {index_file}[/green]")
            
            return True
            
        except Exception as e:
            console.print(f"[red]❌ Ошибка настройки шаблонов: {e}[/red]")
            return False 