"""Основной модуль компилятора GLT."""

import logging
import shutil
import time
from pathlib import Path
from typing import Optional

from jinja2 import Environment, FileSystemLoader, Template
from rich.console import Console
from rich.progress import Progress, SpinnerColumn, TextColumn

from glt_cli.models import GLTConfig
from glt_cli.renderers import (
    DockerRenderer,
    CIRenderer,
    PythonRenderer,
    ReadmeRenderer,
    EnvRenderer,
)
from glt_cli.security import SecurityManager
from glt_cli.cache import TemplateManager
from glt_cli.plugins import PluginManager

console = Console()
logger = logging.getLogger(__name__)


class GLTCompiler:
    """Основной компилятор GLT-проектов."""
    
    def __init__(self, config: GLTConfig):
        """Инициализирует компилятор.
        
        Args:
            config: GLT-конфигурация
        """
        self.config = config
        self.env = Environment(
            loader=FileSystemLoader("templates"),
            autoescape=True,
            trim_blocks=True,
            lstrip_blocks=True,
        )
        
        # Инициализируем рендереры
        self.renderers = {
            "docker": DockerRenderer(config),
            "ci": CIRenderer(config),
            "python": PythonRenderer(config),
            "readme": ReadmeRenderer(config),
            "env": EnvRenderer(config),
        }
        
        # Инициализируем менеджеры
        self.security_manager = SecurityManager(config)
        self.template_manager = TemplateManager(config)
        self.plugin_manager = PluginManager(config)
    
    def compile(self, output_dir: Path, force: bool = False) -> None:
        """Компилирует GLT-проект.
        
        Args:
            output_dir: Директория для вывода
            force: Принудительная перезапись
        """
        logger.info(f"Начинаем компиляцию проекта {self.config.meta.name}")
        
        # Создаем директорию вывода
        output_dir.mkdir(parents=True, exist_ok=True)
        
        # Контекст для плагинов
        context = {
            "project_name": self.config.meta.name,
            "output_dir": str(output_dir),
            "config": self.config,
            "start_time": time.time()
        }
        
        # Выполняем pre_compile хуки плагинов
        self.plugin_manager.execute_hook("on_pre_compile", context)
        
        with Progress(
            SpinnerColumn(),
            TextColumn("[progress.description]{task.description}"),
            console=console,
        ) as progress:
            
            # Задача 1: Генерация структуры проекта
            task1 = progress.add_task("Создание структуры проекта...", total=None)
            self._create_project_structure(output_dir)
            progress.update(task1, completed=True)
            
            # Задача 2: Генерация Python файлов
            task2 = progress.add_task("Генерация Python кода...", total=None)
            self._generate_python_files(output_dir)
            progress.update(task2, completed=True)
            
            # Задача 3: Генерация Docker
            if self.config.docker:
                task3 = progress.add_task("Генерация Docker...", total=None)
                self._generate_docker(output_dir)
                progress.update(task3, completed=True)
            
            # Задача 4: Генерация CI
            if self.config.ci:
                task4 = progress.add_task("Генерация CI...", total=None)
                self._generate_ci(output_dir)
                progress.update(task4, completed=True)
            
            # Задача 5: Генерация README
            task5 = progress.add_task("Генерация README...", total=None)
            self._generate_readme(output_dir)
            progress.update(task5, completed=True)
            
            # Задача 6: Настройка безопасности
            task6 = progress.add_task("Настройка безопасности...", total=None)
            self._setup_security(output_dir)
            progress.update(task6, completed=True)
            
            # Задача 7: Настройка шаблонов
            task7 = progress.add_task("Настройка шаблонов...", total=None)
            self._setup_templates(output_dir)
            progress.update(task7, completed=True)
            
            # Задача 8: Генерация переменных окружения
            task8 = progress.add_task("Генерация переменных окружения...", total=None)
            self._generate_env_files(output_dir)
            progress.update(task8, completed=True)
            
            # Задача 9: Копирование шаблонов
            task9 = progress.add_task("Копирование шаблонов...", total=None)
            self._copy_templates(output_dir)
            progress.update(task9, completed=True)
        
        # Обновляем контекст
        context["duration"] = time.time() - context["start_time"]
        context["end_time"] = time.time()
        
        # Выполняем post_compile хуки плагинов
        self.plugin_manager.execute_hook("on_post_compile", context)
        
        logger.info("Компиляция завершена успешно")
    
    def _create_project_structure(self, output_dir: Path) -> None:
        """Создает базовую структуру проекта."""
        logger.info("Создание структуры проекта")
        
        # Создаем основные директории
        directories = [
            "src",
            "tests",
            "docs",
            "templates",
            ".github/workflows",
            "docker",
        ]
        
        for directory in directories:
            (output_dir / directory).mkdir(parents=True, exist_ok=True)
    
    def _generate_python_files(self, output_dir: Path) -> None:
        """Генерирует Python файлы проекта."""
        logger.info("Генерация Python файлов")
        
        # Генерируем pyproject.toml
        pyproject_content = self.renderers["python"].render_pyproject()
        (output_dir / "pyproject.toml").write_text(pyproject_content, encoding="utf-8")
        
        # Генерируем основной пакет
        package_dir = output_dir / "src" / self.config.meta.name.replace(".", "_")
        package_dir.mkdir(parents=True, exist_ok=True)
        
        # __init__.py
        init_content = self.renderers["python"].render_init()
        (package_dir / "__init__.py").write_text(init_content, encoding="utf-8")
        
        # main.py
        main_content = self.renderers["python"].render_main()
        (package_dir / "main.py").write_text(main_content, encoding="utf-8")
        
        # models.py
        models_content = self.renderers["python"].render_models()
        (package_dir / "models.py").write_text(models_content, encoding="utf-8")
    
    def _generate_docker(self, output_dir: Path) -> None:
        """Генерирует Docker файлы."""
        logger.info("Генерация Docker файлов")
        
        docker_content = self.renderers["docker"].render_dockerfile()
        (output_dir / "docker" / "Dockerfile").write_text(docker_content, encoding="utf-8")
        
        # Docker Compose если нужно
        if self.config.distribution and self.config.distribution.docker_image:
            compose_content = self.renderers["docker"].render_compose()
            (output_dir / "docker-compose.yml").write_text(compose_content, encoding="utf-8")
    
    def _generate_ci(self, output_dir: Path) -> None:
        """Генерирует CI конфигурацию."""
        logger.info("Генерация CI конфигурации")
        
        ci_content = self.renderers["ci"].render_workflow()
        (output_dir / ".github" / "workflows" / "ci.yml").write_text(ci_content, encoding="utf-8")
    
    def _generate_readme(self, output_dir: Path) -> None:
        """Генерирует README файл."""
        logger.info("Генерация README")
        
        readme_content = self.renderers["readme"].render_readme()
        (output_dir / "README.md").write_text(readme_content, encoding="utf-8")
    
    def _setup_security(self, output_dir: Path) -> None:
        """Настраивает безопасность проекта."""
        logger.info("Настройка безопасности")
        
        if self.config.security:
            self.security_manager.setup_security(output_dir)
        else:
            logger.info("Конфигурация безопасности не найдена")
    
    def _setup_templates(self, output_dir: Path) -> None:
        """Настраивает шаблоны проекта."""
        logger.info("Настройка шаблонов")
        
        self.template_manager.setup_templates(output_dir)
    
    def _generate_env_files(self, output_dir: Path) -> None:
        """Генерирует файлы переменных окружения."""
        logger.info("Генерация переменных окружения")
        
        # .envrc для direnv
        envrc_content = self.renderers["env"].render_direnv()
        (output_dir / ".envrc").write_text(envrc_content, encoding="utf-8")
        
        # .env.example
        env_example_content = self.renderers["env"].render_env_example()
        (output_dir / ".env.example").write_text(env_example_content, encoding="utf-8")
        
        # .gitignore дополнения
        gitignore_content = """
# Переменные окружения
.env
.env.local
.env.*.local

# SOPS зашифрованные файлы
*.enc.yaml
*.enc.yml
*.enc.json

# IPFS кэш
.ipfs/
"""
        gitignore_file = output_dir / ".gitignore"
        if gitignore_file.exists():
            gitignore_file.write_text(gitignore_file.read_text() + gitignore_content, encoding="utf-8")
        else:
            gitignore_file.write_text(gitignore_content, encoding="utf-8")
    
    def _copy_templates(self, output_dir: Path) -> None:
        """Копирует шаблоны из исходного проекта."""
        logger.info("Копирование шаблонов")
        
        # Если есть директория templates в исходном проекте
        source_templates = Path("templates")
        if source_templates.exists():
            target_templates = output_dir / "templates"
            if target_templates.exists():
                shutil.rmtree(target_templates)
            shutil.copytree(source_templates, target_templates)
    
    def validate_config(self) -> bool:
        """Проверяет корректность конфигурации."""
        try:
            # Контекст для плагинов
            context = {
                "config": self.config,
                "validation_errors": []
            }
            
            # Проверяем обязательные поля
            if not self.config.meta.name:
                console.print("[red]❌ Имя модуля обязательно[/red]")
                context["validation_errors"].append("Имя модуля обязательно")
                return False
            
            if not self.config.stack.runtime:
                console.print("[red]❌ Runtime обязателен[/red]")
                context["validation_errors"].append("Runtime обязателен")
                return False
            
            # Проверяем версию
            if not self.config.version:
                console.print("[red]❌ Версия обязательна[/red]")
                context["validation_errors"].append("Версия обязательна")
                return False
            
            # Выполняем validate хуки плагинов
            results = self.plugin_manager.execute_hook_with_result("on_validate", context)
            
            # Если хотя бы один плагин вернул False, валидация не прошла
            if False in results:
                return False
            
            console.print("[green]✅ Конфигурация корректна[/green]")
            return True
            
        except Exception as e:
            console.print(f"[red]❌ Ошибка валидации: {e}[/red]")
            return False 