"""Модуль автогенерации документации для GLT CLI."""

import inspect
import logging
from pathlib import Path
from typing import Dict, Any, List, Optional

from rich.console import Console
from rich.table import Table

console = Console()
logger = logging.getLogger(__name__)


class DocsGenerator:
    """Генератор документации для GLT CLI."""
    
    def __init__(self, config: Any):
        """Инициализирует генератор документации.
        
        Args:
            config: GLT конфигурация
        """
        self.config = config
        self.docs_dir = Path("docs")
        self.docs_dir.mkdir(exist_ok=True)
    
    def generate_cli_docs(self) -> str:
        """Генерирует документацию CLI.
        
        Returns:
            str: Путь к сгенерированному файлу
        """
        try:
            console.print("[dim]📚 Генерация CLI документации...[/dim]")
            
            # Импортируем CLI модуль
            from glt_cli.main import app
            
            # Получаем информацию о командах
            commands = self._extract_commands_info(app)
            
            # Генерируем Markdown
            markdown = self._generate_cli_markdown(commands)
            
            # Сохраняем файл
            output_file = self.docs_dir / "cli.md"
            output_file.write_text(markdown, encoding='utf-8')
            
            console.print(f"[green]✅ CLI документация создана: {output_file}[/green]")
            return str(output_file)
            
        except Exception as e:
            logger.error(f"Ошибка генерации CLI документации: {e}")
            console.print(f"[red]❌ Ошибка: {e}[/red]")
            return ""
    
    def generate_api_docs(self) -> str:
        """Генерирует API документацию.
        
        Returns:
            str: Путь к сгенерированному файлу
        """
        try:
            console.print("[dim]📚 Генерация API документации...[/dim]")
            
            # Получаем информацию о модулях
            modules = self._extract_modules_info()
            
            # Генерируем Markdown
            markdown = self._generate_api_markdown(modules)
            
            # Сохраняем файл
            output_file = self.docs_dir / "api.md"
            output_file.write_text(markdown, encoding='utf-8')
            
            console.print(f"[green]✅ API документация создана: {output_file}[/green]")
            return str(output_file)
            
        except Exception as e:
            logger.error(f"Ошибка генерации API документации: {e}")
            console.print(f"[red]❌ Ошибка: {e}[/red]")
            return ""
    
    def generate_project_docs(self) -> str:
        """Генерирует документацию проекта.
        
        Returns:
            str: Путь к сгенерированному файлу
        """
        try:
            console.print("[dim]📚 Генерация документации проекта...[/dim]")
            
            # Генерируем основную документацию
            markdown = self._generate_project_markdown()
            
            # Сохраняем файл
            output_file = self.docs_dir / "project.md"
            output_file.write_text(markdown, encoding='utf-8')
            
            console.print(f"[green]✅ Документация проекта создана: {output_file}[/green]")
            return str(output_file)
            
        except Exception as e:
            logger.error(f"Ошибка генерации документации проекта: {e}")
            console.print(f"[red]❌ Ошибка: {e}[/red]")
            return ""
    
    def generate_all_docs(self) -> List[str]:
        """Генерирует всю документацию.
        
        Returns:
            List[str]: Список созданных файлов
        """
        files = []
        
        # CLI документация
        cli_file = self.generate_cli_docs()
        if cli_file:
            files.append(cli_file)
        
        # API документация
        api_file = self.generate_api_docs()
        if api_file:
            files.append(api_file)
        
        # Документация проекта
        project_file = self.generate_project_docs()
        if project_file:
            files.append(project_file)
        
        # Создаем индекс
        index_file = self._create_docs_index(files)
        if index_file:
            files.append(index_file)
        
        return files
    
    def _extract_commands_info(self, app) -> List[Dict[str, Any]]:
        """Извлекает информацию о командах CLI.
        
        Args:
            app: Typer приложение
            
        Returns:
            List[Dict]: Список информации о командах
        """
        commands = []
        
        try:
            # Получаем команды из Typer приложения
            for command_name, command in app.registered_commands:
                command_info = {
                    'name': command_name,
                    'help': command.help or 'Нет описания',
                    'callback': command.callback,
                    'params': []
                }
                
                # Получаем параметры команды
                if hasattr(command, 'params'):
                    for param in command.params:
                        param_info = {
                            'name': param.name,
                            'type': str(param.type),
                            'help': param.help or 'Нет описания',
                            'required': param.required,
                            'default': param.default
                        }
                        command_info['params'].append(param_info)
                
                commands.append(command_info)
                
        except Exception as e:
            logger.error(f"Ошибка извлечения команд: {e}")
        
        return commands
    
    def _extract_modules_info(self) -> List[Dict[str, Any]]:
        """Извлекает информацию о модулях.
        
        Returns:
            List[Dict]: Список информации о модулях
        """
        modules = []
        
        try:
            # Основные модули GLT CLI
            module_files = [
                ('core', 'core.py'),
                ('models', 'models.py'),
                ('renderers', 'renderers.py'),
                ('utils', 'utils.py'),
                ('security', 'security.py'),
                ('cache', 'cache.py'),
                ('updater', 'updater.py'),
                ('release', 'release.py'),
                ('plugins', 'plugins/__init__.py')
            ]
            
            for module_name, file_name in module_files:
                try:
                    module_path = Path(f"glt_cli/{file_name}")
                    if module_path.exists():
                        module_info = {
                            'name': module_name,
                            'file': file_name,
                            'classes': [],
                            'functions': []
                        }
                        
                        # Извлекаем классы и функции
                        self._extract_module_content(module_path, module_info)
                        modules.append(module_info)
                        
                except Exception as e:
                    logger.error(f"Ошибка извлечения модуля {module_name}: {e}")
                    
        except Exception as e:
            logger.error(f"Ошибка извлечения модулей: {e}")
        
        return modules
    
    def _extract_module_content(self, module_path: Path, module_info: Dict[str, Any]) -> None:
        """Извлекает содержимое модуля.
        
        Args:
            module_path: Путь к модулю
            module_info: Информация о модуле
        """
        try:
            content = module_path.read_text(encoding='utf-8')
            
            # Простой парсинг для поиска классов и функций
            lines = content.split('\n')
            
            for line in lines:
                line = line.strip()
                
                # Классы
                if line.startswith('class ') and ':' in line:
                    class_name = line.split('class ')[1].split('(')[0].split(':')[0].strip()
                    module_info['classes'].append(class_name)
                
                # Функции
                elif line.startswith('def ') and ':' in line:
                    func_name = line.split('def ')[1].split('(')[0].strip()
                    if not func_name.startswith('_'):  # Пропускаем приватные
                        module_info['functions'].append(func_name)
                        
        except Exception as e:
            logger.error(f"Ошибка извлечения содержимого модуля {module_path}: {e}")
    
    def _generate_cli_markdown(self, commands: List[Dict[str, Any]]) -> str:
        """Генерирует Markdown для CLI документации.
        
        Args:
            commands: Список команд
            
        Returns:
            str: Markdown контент
        """
        markdown = f"""# GLT CLI - Документация команд

## Обзор

GLT CLI предоставляет набор команд для работы с GLT-проектами.

**Версия**: {self.config.version}  
**Волна**: {self.config.meta.wave}

## Команды

"""
        
        for command in commands:
            markdown += f"""### {command['name']}

{command['help']}

"""
            
            if command['params']:
                markdown += "**Параметры:**\n\n"
                for param in command['params']:
                    required = "обязательный" if param['required'] else "опциональный"
                    default = f" (по умолчанию: {param['default']})" if param['default'] else ""
                    markdown += f"- `{param['name']}` ({param['type']}, {required}){default} - {param['help']}\n"
                markdown += "\n"
        
        markdown += """## Примеры использования

```bash
# Компиляция проекта
glt compile my-project --output dist/

# Валидация конфигурации
glt validate my-project

# Информация о проекте
glt info my-project

# Управление кэшем
glt cache stats
glt cache clear

# Управление безопасностью
glt security check
glt security encrypt secrets.yaml
```

## Дополнительная информация

Для получения справки по конкретной команде используйте:

```bash
glt <команда> --help
```
"""
        
        return markdown
    
    def _generate_api_markdown(self, modules: List[Dict[str, Any]]) -> str:
        """Генерирует Markdown для API документации.
        
        Args:
            modules: Список модулей
            
        Returns:
            str: Markdown контент
        """
        markdown = f"""# GLT CLI - API Документация

## Обзор

API GLT CLI предоставляет программный интерфейс для работы с GLT-проектами.

**Версия**: {self.config.version}

## Модули

"""
        
        for module in modules:
            markdown += f"""### {module['name']}

**Файл**: `{module['file']}`

"""
            
            if module['classes']:
                markdown += "**Классы:**\n\n"
                for class_name in module['classes']:
                    markdown += f"- `{class_name}`\n"
                markdown += "\n"
            
            if module['functions']:
                markdown += "**Функции:**\n\n"
                for func_name in module['functions']:
                    markdown += f"- `{func_name}`\n"
                markdown += "\n"
        
        markdown += """## Основные классы

### GLTCompiler

Основной компилятор GLT-проектов.

```python
from glt_cli.core import GLTCompiler
from glt_cli.utils import load_glt_config

# Загружаем конфигурацию
config = load_glt_config(Path("my-project"))

# Создаем компилятор
compiler = GLTCompiler(config)

# Компилируем проект
compiler.compile(Path("output"))
```

### PluginManager

Менеджер плагинов для расширения функциональности.

```python
from glt_cli.plugins import PluginManager

# Создаем менеджер плагинов
plugin_manager = PluginManager(config)

# Регистрируем плагин
plugin_manager.register_plugin("my_plugin", MyPlugin(config))

# Выполняем хук
plugin_manager.execute_hook("on_pre_compile", context)
```

## Примеры использования

### Создание собственного плагина

```python
from glt_cli.plugins import BasePlugin

class MyPlugin(BasePlugin):
    def get_name(self) -> str:
        return "my_plugin"
    
    def get_version(self) -> str:
        return "1.0.0"
    
    def on_pre_compile(self, context: Dict[str, Any]) -> None:
        print("Мой плагин выполняется перед компиляцией")
```

### Работа с кэшем

```python
from glt_cli.cache import IPFSCache

# Создаем кэш
cache = IPFSCache(config)

# Получаем шаблон
template = cache.get_template("template_hash")

# Кэшируем шаблон
cache.cache_template("template_hash", template_content)
```
"""
        
        return markdown
    
    def _generate_project_markdown(self) -> str:
        """Генерирует Markdown для документации проекта.
        
        Returns:
            str: Markdown контент
        """
        markdown = f"""# {self.config.meta.name} - Документация проекта

## Обзор

**Версия**: {self.config.version}  
**Волна**: {self.config.meta.wave}  
**Владелец**: {self.config.meta.owner}  
**Видимость**: {self.config.meta.visibility}

## Архитектура

### Структура проекта

```
{self.config.meta.name}/
├── src/
│   └── {self.config.meta.name.replace('.', '_')}/
│       ├── __init__.py
│       ├── main.py
│       └── models.py
├── tests/
├── docs/
├── docker/
├── .github/
│   └── workflows/
└── templates/
```

### Технологический стек

**Runtime**: {self.config.stack.runtime}

**Зависимости:**
"""
        
        for dep in self.config.stack.deps:
            markdown += f"- {dep}\n"
        
        markdown += "\n**Dev зависимости:**\n"
        for dep in self.config.stack.dev_deps:
            markdown += f"- {dep}\n"
        
        markdown += f"""
### Alien Tech

**Технология**: {self.config.alien_tech or 'Не указана'}

### Законы

"""
        
        for law in self.config.laws:
            markdown += f"- {law}\n"
        
        markdown += f"""
## Конфигурация

### Репозиторий

- **URL**: {self.config.repo.url}
- **Защита веток**: {'Да' if self.config.repo.branch_protect else 'Нет'}
- **GPG подпись**: {'Да' if self.config.repo.gpg_sign else 'Нет'}
- **Семантические релизы**: {'Да' if self.config.repo.semantic_release else 'Нет'}

### Распространение

- **PyPI Registry**: {self.config.distribution.pypi_registry}
- **Автопубликация**: {'Да' if self.config.distribution.auto_publish else 'Нет'}
- **Docker Image**: {self.config.distribution.docker_image}

### Инфраструктура

- **IPFS Cache**: {self.config.infra.cache_ipfs}
- **Observability**: {self.config.infra.observability}

### Безопасность

- **SOPS**: {'Да' if self.config.security.sops else 'Нет'}
- **Pre-commit**: {', '.join(self.config.security.pre_commit) if self.config.security.pre_commit else 'Нет'}

## Разработка

### Установка

```bash
# Клонирование репозитория
git clone {self.config.repo.url}

# Установка зависимостей
pip install -r requirements.txt

# Установка в режиме разработки
pip install -e .
```

### Тестирование

```bash
# Запуск тестов
pytest tests/

# Покрытие кода
pytest --cov=src tests/

# Линтинг
ruff check .
```

### Сборка

```bash
# Сборка пакета
python -m build

# Создание Docker образа
docker build -t {self.config.distribution.docker_image} .
```

## Деплой

### Автоматический деплой

Проект настроен для автоматического деплоя через GitHub Actions.

### Ручной деплой

```bash
# Публикация в PyPI
twine upload dist/*

# Публикация Docker образа
docker push {self.config.distribution.docker_image}
```

## Мониторинг

### Телеметрия

- **Формат логов**: {self.config.telemetry.log_format}
- **Интервал отправки**: {self.config.telemetry.push_interval}
- **Grafana Dashboard**: {self.config.telemetry.grafana_dashboard}

## Поддержка

Для получения поддержки обратитесь к владельцу проекта: {self.config.meta.owner}
"""
        
        return markdown
    
    def _create_docs_index(self, files: List[str]) -> Optional[str]:
        """Создает индекс документации.
        
        Args:
            files: Список созданных файлов
            
        Returns:
            str: Путь к индексному файлу
        """
        try:
            markdown = f"""# {self.config.meta.name} - Документация

## Обзор

Документация для проекта {self.config.meta.name} (версия {self.config.version}).

## Разделы

"""
            
            # Добавляем ссылки на файлы
            file_names = {
                'cli.md': 'CLI Команды',
                'api.md': 'API Документация', 
                'project.md': 'Документация проекта'
            }
            
            for file_path in files:
                file_name = Path(file_path).name
                if file_name in file_names:
                    markdown += f"- [{file_names[file_name]}]({file_name})\n"
            
            markdown += f"""
## Быстрый старт

1. [CLI Команды](cli.md) - Основные команды для работы с проектом
2. [API Документация](api.md) - Программный интерфейс
3. [Документация проекта](project.md) - Подробная информация о проекте

## Генерация документации

Документация автоматически генерируется командой:

```bash
glt docs generate
```

**Последнее обновление**: {self.config.version}
"""
            
            # Сохраняем индекс
            output_file = self.docs_dir / "index.md"
            output_file.write_text(markdown, encoding='utf-8')
            
            console.print(f"[green]✅ Индекс документации создан: {output_file}[/green]")
            return str(output_file)
            
        except Exception as e:
            logger.error(f"Ошибка создания индекса документации: {e}")
            return None 