"""Основной модуль GLT CLI."""

import sys
from pathlib import Path
from typing import Optional

import typer
from rich.console import Console
from rich.traceback import install

from glt_cli.config import setup_logging
from glt_cli.core import GLTCompiler
from glt_cli.utils import load_glt_config

# Устанавливаем rich traceback для лучшего отображения ошибок
install(show_locals=True)

app = typer.Typer(
    name="glt",
    help="CLI-инструмент для компиляции GLT-проектов",
    add_completion=False,
    rich_markup_mode="rich",
)

console = Console()


@app.command()
def compile(
    project_path: Optional[Path] = typer.Argument(
        None,
        help="Путь к GLT-проекту (по умолчанию текущая директория)",
        exists=True,
        file_okay=False,
        dir_okay=True,
    ),
    output_dir: Optional[Path] = typer.Option(
        None,
        "--output",
        "-o",
        help="Директория для вывода скомпилированного проекта",
    ),
    force: bool = typer.Option(
        False,
        "--force",
        "-f",
        help="Принудительная перезапись существующих файлов",
    ),
    verbose: bool = typer.Option(
        False,
        "--verbose",
        "-v",
        help="Подробный вывод",
    ),
) -> None:
    """Компилирует GLT-проект в готовый репозиторий."""
    try:
        # Настройка логирования
        setup_logging(verbose=verbose)
        
        # Определяем путь к проекту
        if project_path is None:
            project_path = Path.cwd()
        
        # Определяем директорию вывода
        if output_dir is None:
            output_dir = project_path / "dist"
        
        console.print(f"[bold blue]🔧 Компиляция GLT-проекта[/bold blue]")
        console.print(f"📁 Проект: {project_path}")
        console.print(f"📤 Вывод: {output_dir}")
        
        # Загружаем конфигурацию GLT
        glt_config = load_glt_config(project_path)
        
        # Создаем компилятор и запускаем процесс
        compiler = GLTCompiler(glt_config)
        compiler.compile(output_dir=output_dir, force=force)
        
        console.print("[bold green]✅ Компиляция завершена успешно![/bold green]")
        
    except Exception as e:
        console.print(f"[bold red]❌ Ошибка компиляции: {e}[/bold red]")
        if verbose:
            console.print_exception()
        sys.exit(1)


@app.command()
def validate(
    project_path: Optional[Path] = typer.Argument(
        None,
        help="Путь к GLT-проекту (по умолчанию текущая директория)",
        exists=True,
        file_okay=False,
        dir_okay=True,
    ),
) -> None:
    """Проверяет корректность GLT-конфигурации."""
    try:
        if project_path is None:
            project_path = Path.cwd()
        
        console.print(f"[bold blue]🔍 Валидация GLT-проекта[/bold blue]")
        console.print(f"📁 Проект: {project_path}")
        
        # Загружаем и валидируем конфигурацию
        glt_config = load_glt_config(project_path)
        
        console.print("[bold green]✅ Конфигурация корректна![/bold green]")
        console.print(f"📋 Версия схемы: {glt_config.version}")
        console.print(f"🏷️  Имя модуля: {glt_config.meta.name}")
        
    except Exception as e:
        console.print(f"[bold red]❌ Ошибка валидации: {e}[/bold red]")
        sys.exit(1)


@app.command()
def cache(
    action: str = typer.Argument(
        "stats",
        help="Действие с кэшем (stats/list/clear)",
    ),
    project_path: Optional[Path] = typer.Argument(
        None,
        help="Путь к GLT-проекту (по умолчанию текущая директория)",
        exists=True,
        file_okay=False,
        dir_okay=True,
    ),
) -> None:
    """Управление IPFS кэшем шаблонов."""
    try:
        if project_path is None:
            project_path = Path.cwd()
        
        # Загружаем конфигурацию
        glt_config = load_glt_config(project_path)
        
        # Инициализируем кэш
        from glt_cli.cache import IPFSCache
        ipfs_cache = IPFSCache(glt_config)
        
        if action == "stats":
            stats = ipfs_cache.get_cache_stats()
            from rich.table import Table
            
            table = Table(title="IPFS Кэш - Статистика")
            table.add_column("Параметр", style="cyan")
            table.add_column("Значение", style="magenta")
            
            for key, value in stats.items():
                table.add_row(key, str(value))
            
            console.print(table)
            
        elif action == "list":
            templates = ipfs_cache.list_cached_templates()
            from rich.table import Table
            
            table = Table(title="IPFS Кэш - Шаблоны")
            table.add_column("Хэш", style="cyan")
            table.add_column("Размер", style="magenta")
            table.add_column("Alien Tech", style="green")
            
            for template in templates:
                size_mb = round(template.get("size", 0) / 1024 / 1024, 2)
                table.add_row(
                    template.get("hash", "N/A"),
                    f"{size_mb} MB",
                    template.get("alien_tech", "N/A")
                )
            
            console.print(table)
            
        elif action == "clear":
            if ipfs_cache.clear_cache():
                console.print("[green]✅ Кэш очищен[/green]")
            else:
                console.print("[red]❌ Ошибка очистки кэша[/red]")
        else:
            console.print(f"[red]❌ Неизвестное действие: {action}[/red]")
            console.print("Доступные действия: stats, list, clear")
            
    except Exception as e:
        console.print(f"[red]❌ Ошибка: {e}[/red]")
        sys.exit(1)


@app.command()
def security(
    action: str = typer.Argument(
        "check",
        help="Действие с безопасностью (check/encrypt/decrypt)",
    ),
    file_path: Optional[Path] = typer.Argument(
        None,
        help="Путь к файлу для шифрования/расшифрования",
    ),
    project_path: Optional[Path] = typer.Argument(
        None,
        help="Путь к GLT-проекту (по умолчанию текущая директория)",
        exists=True,
        file_okay=False,
        dir_okay=True,
    ),
) -> None:
    """Управление безопасностью (SOPS)."""
    try:
        if project_path is None:
            project_path = Path.cwd()
        
        # Загружаем конфигурацию
        glt_config = load_glt_config(project_path)
        
        # Инициализируем менеджер безопасности
        from glt_cli.security import SOPSManager
        sops_manager = SOPSManager(glt_config)
        
        if action == "check":
            if sops_manager.is_sops_available():
                console.print("[green]✅ SOPS доступен[/green]")
            else:
                console.print("[red]❌ SOPS недоступен[/red]")
                
        elif action == "encrypt":
            if not file_path:
                console.print("[red]❌ Укажите путь к файлу для шифрования[/red]")
                return
            
            if sops_manager.encrypt_file(file_path):
                console.print("[green]✅ Файл зашифрован[/green]")
            else:
                console.print("[red]❌ Ошибка шифрования[/red]")
                
        elif action == "decrypt":
            if not file_path:
                console.print("[red]❌ Укажите путь к файлу для расшифрования[/red]")
                return
            
            if sops_manager.decrypt_file(file_path):
                console.print("[green]✅ Файл расшифрован[/green]")
            else:
                console.print("[red]❌ Ошибка расшифрования[/red]")
        else:
            console.print(f"[red]❌ Неизвестное действие: {action}[/red]")
            console.print("Доступные действия: check, encrypt, decrypt")
            
    except Exception as e:
        console.print(f"[red]❌ Ошибка: {e}[/red]")
        sys.exit(1)


@app.command()
def plugins(
    action: str = typer.Argument(
        "list",
        help="Действие с плагинами (list/info/load)",
    ),
    project_path: Optional[Path] = typer.Argument(
        None,
        help="Путь к GLT-проекту (по умолчанию текущая директория)",
        exists=True,
        file_okay=False,
        dir_okay=True,
    ),
) -> None:
    """Управление плагинами GLT CLI."""
    try:
        if project_path is None:
            project_path = Path.cwd()
        
        # Загружаем конфигурацию
        glt_config = load_glt_config(project_path)
        
        # Инициализируем менеджер плагинов
        from glt_cli.plugins import PluginManager
        plugin_manager = PluginManager(glt_config)
        
        if action == "list":
            plugins = plugin_manager.list_plugins()
            from rich.table import Table
            
            table = Table(title="Плагины GLT CLI")
            table.add_column("Имя", style="cyan")
            table.add_column("Класс", style="magenta")
            table.add_column("Версия", style="green")
            table.add_column("Хуки", style="yellow")
            
            for plugin in plugins:
                hooks_str = ", ".join(plugin.get("hooks", []))
                table.add_row(
                    plugin.get("name", "N/A"),
                    plugin.get("class", "N/A"),
                    plugin.get("version", "N/A"),
                    hooks_str
                )
            
            console.print(table)
            
        elif action == "info":
            plugins = plugin_manager.list_plugins()
            console.print(f"[blue]📊 Всего плагинов: {len(plugins)}[/blue]")
            
            for plugin in plugins:
                console.print(f"[cyan]🔌 {plugin.get('name')}[/cyan]")
                console.print(f"  Класс: {plugin.get('class')}")
                console.print(f"  Версия: {plugin.get('version')}")
                console.print(f"  Хуки: {', '.join(plugin.get('hooks', []))}")
                console.print()
                
        elif action == "load":
            plugin_manager.load_external_plugins()
            console.print("[green]✅ Внешние плагины загружены[/green]")
        else:
            console.print(f"[red]❌ Неизвестное действие: {action}[/red]")
            console.print("Доступные действия: list, info, load")
            
    except Exception as e:
        console.print(f"[red]❌ Ошибка: {e}[/red]")
        sys.exit(1)


@app.command()
def update(
    target_version: Optional[str] = typer.Argument(
        None,
        help="Целевая версия для обновления",
    ),
    project_path: Optional[Path] = typer.Argument(
        None,
        help="Путь к GLT-проекту (по умолчанию текущая директория)",
        exists=True,
        file_okay=False,
        dir_okay=True,
    ),
) -> None:
    """Обновление GLT CLI."""
    try:
        if project_path is None:
            project_path = Path.cwd()
        
        # Загружаем конфигурацию
        glt_config = load_glt_config(project_path)
        
        # Инициализируем автообновление
        from glt_cli.updater import AutoUpdater
        updater = AutoUpdater(glt_config)
        
        if target_version:
            success = updater.update(target_version)
        else:
            success = updater.update()
        
        if not success:
            sys.exit(1)
            
    except Exception as e:
        console.print(f"[red]❌ Ошибка: {e}[/red]")
        sys.exit(1)


@app.command()
def release(
    action: str = typer.Argument(
        "analyze",
        help="Действие с релизами (analyze/create/info)",
    ),
    version: Optional[str] = typer.Argument(
        None,
        help="Версия для релиза",
    ),
    dry_run: bool = typer.Option(
        False,
        "--dry-run",
        help="Тестовый запуск без создания релиза",
    ),
    project_path: Optional[Path] = typer.Argument(
        None,
        help="Путь к GLT-проекту (по умолчанию текущая директория)",
        exists=True,
        file_okay=False,
        dir_okay=True,
    ),
) -> None:
    """Управление семантическими релизами."""
    try:
        if project_path is None:
            project_path = Path.cwd()
        
        # Загружаем конфигурацию
        glt_config = load_glt_config(project_path)
        
        # Инициализируем семантические релизы
        from glt_cli.release import SemanticRelease
        semantic_release = SemanticRelease(glt_config)
        
        if action == "analyze":
            new_version = semantic_release.analyze_commits()
            console.print(f"[green]📈 Рекомендуемая версия: {new_version}[/green]")
            
        elif action == "create":
            if not version:
                version = semantic_release.analyze_commits()
            
            success = semantic_release.create_release(version, dry_run)
            if not success:
                sys.exit(1)
                
        elif action == "info":
            semantic_release.show_release_info()
        else:
            console.print(f"[red]❌ Неизвестное действие: {action}[/red]")
            console.print("Доступные действия: analyze, create, info")
            
    except Exception as e:
        console.print(f"[red]❌ Ошибка: {e}[/red]")
        sys.exit(1)


@app.command()
def docs(
    action: str = typer.Argument(
        "generate",
        help="Действие с документацией (generate/cli/api/project)",
    ),
    project_path: Optional[Path] = typer.Argument(
        None,
        help="Путь к GLT-проекту (по умолчанию текущая директория)",
        exists=True,
        file_okay=False,
        dir_okay=True,
    ),
) -> None:
    """Генерация документации."""
    try:
        if project_path is None:
            project_path = Path.cwd()
        
        # Загружаем конфигурацию
        glt_config = load_glt_config(project_path)
        
        # Инициализируем генератор документации
        from glt_cli.docs import DocsGenerator
        docs_generator = DocsGenerator(glt_config)
        
        if action == "generate":
            files = docs_generator.generate_all_docs()
            console.print(f"[green]✅ Создано файлов документации: {len(files)}[/green]")
            
        elif action == "cli":
            file = docs_generator.generate_cli_docs()
            if file:
                console.print(f"[green]✅ CLI документация создана: {file}[/green]")
                
        elif action == "api":
            file = docs_generator.generate_api_docs()
            if file:
                console.print(f"[green]✅ API документация создана: {file}[/green]")
                
        elif action == "project":
            file = docs_generator.generate_project_docs()
            if file:
                console.print(f"[green]✅ Документация проекта создана: {file}[/green]")
        else:
            console.print(f"[red]❌ Неизвестное действие: {action}[/red]")
            console.print("Доступные действия: generate, cli, api, project")
            
    except Exception as e:
        console.print(f"[red]❌ Ошибка: {e}[/red]")
        sys.exit(1)


@app.command()
def patch(
    patch_file: Path = typer.Argument(
        ...,
        help="Путь к файлу патча (.glt)",
        exists=True,
        file_okay=True,
        dir_okay=False,
    ),
    project_path: Optional[Path] = typer.Argument(
        None,
        help="Путь к GLT-проекту (по умолчанию текущая директория)",
        exists=True,
        file_okay=False,
        dir_okay=True,
    ),
    dry_run: bool = typer.Option(
        False,
        "--dry-run",
        help="Тестовый запуск без применения изменений",
    ),
) -> None:
    """Применяет патч к GLT-проекту."""
    try:
        if project_path is None:
            project_path = Path.cwd()
        
        console.print(f"[bold blue]🔧 Применение патча[/bold blue]")
        console.print(f"📁 Проект: {project_path}")
        console.print(f"🔧 Патч: {patch_file}")
        
        # Загружаем патч
        from glt_cli.patch import PatchApplier
        patch_applier = PatchApplier(project_path)
        
        if dry_run:
            console.print("[yellow]🔍 Тестовый режим - изменения не применяются[/yellow]")
            changes = patch_applier.analyze_patch(patch_file)
            console.print(f"[green]✅ Проанализировано изменений: {len(changes)}[/green]")
        else:
            changes = patch_applier.apply_patch(patch_file)
            console.print(f"[green]✅ Патч применен успешно! Изменений: {len(changes)}[/green]")
        
    except Exception as e:
        console.print(f"[bold red]❌ Ошибка применения патча: {e}[/bold red]")
        sys.exit(1)


@app.command()
def info(
    project_path: Optional[Path] = typer.Argument(
        None,
        help="Путь к GLT-проекту (по умолчанию текущая директория)",
        exists=True,
        file_okay=False,
        dir_okay=True,
    ),
) -> None:
    """Показывает информацию о GLT-проекте."""
    try:
        if project_path is None:
            project_path = Path.cwd()
        
        console.print(f"[bold blue]ℹ️  Информация о GLT-проекте[/bold blue]")
        console.print(f"📁 Проект: {project_path}")
        
        # Загружаем конфигурацию
        glt_config = load_glt_config(project_path)
        
        # Выводим информацию
        from rich.table import Table
        
        table = Table(title="GLT Конфигурация")
        table.add_column("Параметр", style="cyan")
        table.add_column("Значение", style="magenta")
        
        table.add_row("Версия схемы", str(glt_config.version))
        table.add_row("Имя модуля", glt_config.meta.name)
        table.add_row("Волна", str(glt_config.meta.wave))
        table.add_row("Владелец", glt_config.meta.owner)
        table.add_row("Видимость", glt_config.meta.visibility)
        table.add_row("Runtime", glt_config.stack.runtime)
        
        console.print(table)
        
    except Exception as e:
        console.print(f"[bold red]❌ Ошибка: {e}[/bold red]")
        sys.exit(1)


def main() -> None:
    """Точка входа в приложение."""
    app()


if __name__ == "__main__":
    main() 