"""Модели данных для GLT-конфигурации."""

from typing import Any, Dict, List, Optional, Union

from pydantic import BaseModel, Field, field_validator


class MetaInfo(BaseModel):
    """Метаинформация о GLT-модуле."""
    
    name: str = Field(..., description="Уникальное имя модуля")
    wave: str = Field(..., description="Волна разработки (например, Æ3)")
    owner: str = Field(..., description="Владелец модуля")
    visibility: str = Field(..., description="Видимость (public/private)")
    
    @field_validator("visibility")
    @classmethod
    def validate_visibility(cls, v: str) -> str:
        """Проверяет корректность видимости."""
        if v not in ["public", "private"]:
            raise ValueError("Видимость должна быть 'public' или 'private'")
        return v


class Repository(BaseModel):
    """Конфигурация репозитория."""
    
    url: str = Field(..., description="URL репозитория")
    branch_protect: bool = Field(default=True, description="Защита веток")
    gpg_sign: bool = Field(default=True, description="GPG подпись")
    semantic_release: bool = Field(default=True, description="Семантические релизы")


class Distribution(BaseModel):
    """Конфигурация дистрибуции."""
    
    pypi_registry: Optional[str] = Field(None, description="PyPI registry")
    auto_publish: bool = Field(default=True, description="Автопубликация")
    docker_image: Optional[str] = Field(None, description="Docker образ")


class Stack(BaseModel):
    """Стек технологий."""
    
    runtime: str = Field(..., description="Runtime (например, python:3.12-slim)")
    deps: List[str] = Field(default_factory=list, description="Основные зависимости")
    dev_deps: List[str] = Field(default_factory=list, description="Dev зависимости")


class Infrastructure(BaseModel):
    """Инфраструктурные сервисы."""
    
    cache_ipfs: Optional[str] = Field(None, description="IPFS кэш")
    observability: Optional[str] = Field(None, description="Observability stack")


class Security(BaseModel):
    """Конфигурация безопасности."""
    
    sops: Optional[Dict[str, Any]] = Field(None, description="SOPS конфигурация")
    pre_commit: List[str] = Field(default_factory=list, description="Pre-commit хуки")


class Telemetry(BaseModel):
    """Конфигурация телеметрии."""
    
    grafana_dashboard: Optional[str] = Field(None, description="Grafana dashboard")
    log_format: str = Field(default="ecs-json", description="Формат логов")
    push_interval: str = Field(default="30s", description="Интервал отправки")


class Plan(BaseModel):
    """План разработки."""
    
    wave: str = Field(..., description="Волна (например, Φ-13)")
    tasks: List[str] = Field(..., description="Список задач")


class GLTConfig(BaseModel):
    """Основная конфигурация GLT-проекта."""
    
    version: Union[str, float] = Field(..., description="Версия схемы GLT")
    meta: MetaInfo = Field(..., description="Метаинформация")
    repo: Optional[Repository] = Field(None, description="Конфигурация репозитория")
    distribution: Optional[Distribution] = Field(None, description="Дистрибуция")
    stack: Stack = Field(..., description="Стек технологий")
    alien_tech: Optional[str] = Field(None, description="Инопланетные технологии")
    laws: List[str] = Field(default_factory=list, description="Законы природы")
    infra: Optional[Infrastructure] = Field(None, description="Инфраструктура")
    secrets: List[str] = Field(default_factory=list, description="Секреты")
    routes: List[str] = Field(default_factory=list, description="Маршруты")
    plan: List[Plan] = Field(default_factory=list, description="План разработки")
    docker: Optional[str] = Field(None, description="Путь к Dockerfile")
    ci: Optional[str] = Field(None, description="Путь к CI конфигурации")
    security: Optional[Security] = Field(None, description="Безопасность")
    telemetry: Optional[Telemetry] = Field(None, description="Телеметрия")
    
    @field_validator("version")
    @classmethod
    def validate_version(cls, v: Union[str, float]) -> str:
        """Проверяет корректность версии."""
        # Преобразуем в строку
        version_str = str(v)
        
        # Проверяем формат версии (например, 0.6, 1.0, 2.1.3)
        import re
        if not re.match(r'^\d+(\.\d+)*$', version_str):
            raise ValueError(f"Неверный формат версии: {version_str}")
        
        return version_str
    
    @field_validator("meta")
    @classmethod
    def validate_meta(cls, v: MetaInfo) -> MetaInfo:
        """Проверяет метаинформацию."""
        if v.visibility not in ["public", "private"]:
            raise ValueError("Видимость должна быть 'public' или 'private'")
        return v
    
    @field_validator("stack")
    @classmethod
    def validate_stack(cls, v: Stack) -> Stack:
        """Проверяет стек технологий."""
        if not v.runtime:
            raise ValueError("Runtime обязателен")
        return v 