"""Модуль для применения патчей к GLT проектам."""

import logging
from pathlib import Path
from typing import Dict, Any, List, Optional

import ruamel.yaml
from rich.console import Console

from glt_cli.utils import load_glt_config

console = Console()
logger = logging.getLogger(__name__)


class PatchApplier:
    """Применяет патчи к GLT проектам."""
    
    def __init__(self, project_path: Path):
        """Инициализация.
        
        Args:
            project_path: Путь к проекту
        """
        self.project_path = project_path
        self.config_file = self._find_config_file()
        
    def _find_config_file(self) -> Path:
        """Находит файл конфигурации."""
        # Ищем .glt файл
        glt_file = self.project_path / ".glt"
        if glt_file.exists() and glt_file.is_file():
            return glt_file
            
        # Ищем файлы с расширением .glt
        glt_files = list(self.project_path.glob("*.glt"))
        if glt_files:
            return glt_files[0]
            
        raise FileNotFoundError(f"Файл конфигурации не найден в {self.project_path}")
    
    def load_patch(self, patch_file: Path) -> Dict[str, Any]:
        """Загружает патч из файла.
        
        Args:
            patch_file: Путь к файлу патча
            
        Returns:
            Содержимое патча
        """
        try:
            yaml = ruamel.yaml.YAML(typ='safe')
            with open(patch_file, 'r', encoding='utf-8') as f:
                data = yaml.load(f)
            
            if not data:
                raise ValueError("Файл патча пуст")
                
            return data
        except ruamel.yaml.YAMLError as e:
            raise ValueError(f"Ошибка парсинга YAML: {e}")
    
    def load_config(self) -> Dict[str, Any]:
        """Загружает текущую конфигурацию.
        
        Returns:
            Текущая конфигурация
        """
        try:
            yaml = ruamel.yaml.YAML(typ='safe')
            with open(self.config_file, 'r', encoding='utf-8') as f:
                data = yaml.load(f)
            
            return data or {}
        except Exception as e:
            raise ValueError(f"Ошибка загрузки конфигурации: {e}")
    
    def save_config(self, config: Dict[str, Any]) -> None:
        """Сохраняет конфигурацию.
        
        Args:
            config: Конфигурация для сохранения
        """
        try:
            yaml = ruamel.yaml.YAML(typ='safe')
            with open(self.config_file, 'w', encoding='utf-8') as f:
                yaml.dump(config, f)
        except Exception as e:
            raise ValueError(f"Ошибка сохранения конфигурации: {e}")
    
    def apply_replace(self, config: Dict[str, Any], path: str, value: Any) -> None:
        """Применяет замену.
        
        Args:
            config: Конфигурация
            path: Путь к заменяемому значению
            value: Новое значение
        """
        keys = path.split('.')
        current = config
        
        # Проходим по пути до предпоследнего элемента
        for key in keys[:-1]:
            if key not in current:
                current[key] = {}
            current = current[key]
        
        # Заменяем последний элемент
        current[keys[-1]] = value
    
    def apply_merge(self, config: Dict[str, Any], path: str, value: Any) -> None:
        """Применяет слияние.
        
        Args:
            config: Конфигурация
            path: Путь к сливаемому значению
            value: Значение для слияния
        """
        keys = path.split('.')
        current = config
        
        # Проходим по пути до предпоследнего элемента
        for key in keys[:-1]:
            if key not in current:
                current[key] = {}
            current = current[key]
        
        # Сливаем последний элемент
        if keys[-1] not in current:
            current[keys[-1]] = {}
        
        if isinstance(value, dict) and isinstance(current[keys[-1]], dict):
            current[keys[-1]].update(value)
        else:
            current[keys[-1]] = value
    
    def apply_delete(self, config: Dict[str, Any], path: str, key: str) -> None:
        """Применяет удаление.
        
        Args:
            config: Конфигурация
            path: Путь к удаляемому значению
            key: Ключ для удаления
        """
        keys = path.split('.')
        current = config
        
        # Проходим по пути
        for k in keys:
            if k not in current:
                return  # Элемент не существует
            current = current[k]
        
        # Удаляем ключ
        if isinstance(current, dict) and key in current:
            del current[key]
    
    def apply_append(self, config: Dict[str, Any], path: str, value: Any) -> None:
        """Применяет добавление.
        
        Args:
            config: Конфигурация
            path: Путь к списку
            value: Значение для добавления
        """
        keys = path.split('.')
        current = config
        
        # Проходим по пути
        for key in keys:
            if key not in current:
                current[key] = []
            current = current[key]
        
        # Добавляем значение
        if isinstance(current, list):
            if isinstance(value, list):
                current.extend(value)
            else:
                current.append(value)
    
    def analyze_patch(self, patch_file: Path) -> List[Dict[str, Any]]:
        """Анализирует патч без применения.
        
        Args:
            patch_file: Путь к файлу патча
            
        Returns:
            Список изменений
        """
        patch_data = self.load_patch(patch_file)
        changes = []
        
        for action, data in patch_data.items():
            if action.startswith('replace'):
                changes.append({
                    'type': 'replace',
                    'path': data.get('path'),
                    'value': data.get('value')
                })
            elif action.startswith('merge'):
                changes.append({
                    'type': 'merge',
                    'path': data.get('path'),
                    'value': data.get('value')
                })
            elif action.startswith('delete'):
                changes.append({
                    'type': 'delete',
                    'path': data.get('path'),
                    'key': data.get('key')
                })
            elif action.startswith('append'):
                changes.append({
                    'type': 'append',
                    'path': data.get('path'),
                    'value': data.get('value')
                })
        
        return changes
    
    def apply_patch(self, patch_file: Path) -> List[Dict[str, Any]]:
        """Применяет патч к проекту.
        
        Args:
            patch_file: Путь к файлу патча
            
        Returns:
            Список примененных изменений
        """
        patch_data = self.load_patch(patch_file)
        config = self.load_config()
        changes = []
        
        console.print(f"[blue]🔧 Применение патча: {patch_file.name}[/blue]")
        
        for action, data in patch_data.items():
            if action.startswith('replace'):
                path = data.get('path')
                value = data.get('value')
                if path and 'value' in data:
                    self.apply_replace(config, path, value)
                    changes.append({'type': 'replace', 'path': path})
                    console.print(f"  ✅ Заменено: {path}")
                    
            elif action.startswith('merge'):
                path = data.get('path')
                value = data.get('value')
                if path and 'value' in data:
                    self.apply_merge(config, path, value)
                    changes.append({'type': 'merge', 'path': path})
                    console.print(f"  ✅ Слито: {path}")
                    
            elif action.startswith('delete'):
                path = data.get('path')
                key = data.get('key')
                if path and 'key' in data:
                    self.apply_delete(config, path, key)
                    changes.append({'type': 'delete', 'path': path, 'key': key})
                    console.print(f"  ✅ Удалено: {path}.{key}")
                    
            elif action.startswith('append'):
                path = data.get('path')
                value = data.get('value')
                if path and 'value' in data:
                    self.apply_append(config, path, value)
                    changes.append({'type': 'append', 'path': path})
                    console.print(f"  ✅ Добавлено: {path}")
        
        # Сохраняем измененную конфигурацию
        self.save_config(config)
        
        return changes 