"""Архитектура плагинов для GLT CLI."""

import importlib
import inspect
import logging
from abc import ABC, abstractmethod
from pathlib import Path
from typing import Dict, Any, List, Optional, Type

from rich.console import Console

console = Console()
logger = logging.getLogger(__name__)


class BasePlugin(ABC):
    """Базовый класс для всех плагинов GLT CLI."""
    
    def __init__(self, config: Any):
        """Инициализирует плагин.
        
        Args:
            config: GLT конфигурация
        """
        self.config = config
        self.name = self.__class__.__name__
        self.version = getattr(self, 'version', '1.0.0')
    
    @abstractmethod
    def get_name(self) -> str:
        """Возвращает имя плагина."""
        pass
    
    @abstractmethod
    def get_version(self) -> str:
        """Возвращает версию плагина."""
        pass
    
    def on_pre_compile(self, context: Dict[str, Any]) -> None:
        """Вызывается перед компиляцией.
        
        Args:
            context: Контекст компиляции
        """
        logger.debug(f"Плагин {self.name}: pre_compile")
    
    def on_post_compile(self, context: Dict[str, Any]) -> None:
        """Вызывается после компиляции.
        
        Args:
            context: Контекст компиляции
        """
        logger.debug(f"Плагин {self.name}: post_compile")
    
    def on_validate(self, context: Dict[str, Any]) -> bool:
        """Вызывается при валидации.
        
        Args:
            context: Контекст валидации
            
        Returns:
            bool: True если валидация прошла успешно
        """
        logger.debug(f"Плагин {self.name}: validate")
        return True
    
    def on_error(self, error: Exception, context: Dict[str, Any]) -> None:
        """Вызывается при ошибке.
        
        Args:
            error: Исключение
            context: Контекст ошибки
        """
        logger.error(f"Плагин {self.name}: error - {error}")


class PluginManager:
    """Менеджер плагинов GLT CLI."""
    
    def __init__(self, config: Any):
        """Инициализирует менеджер плагинов.
        
        Args:
            config: GLT конфигурация
        """
        self.config = config
        self.plugins: Dict[str, BasePlugin] = {}
        self.plugins_dir = Path("plugins")
        
        # Автоматически загружаем встроенные плагины
        self._load_builtin_plugins()
    
    def register_plugin(self, name: str, plugin: BasePlugin) -> None:
        """Регистрирует плагин.
        
        Args:
            name: Имя плагина
            plugin: Экземпляр плагина
        """
        self.plugins[name] = plugin
        logger.info(f"Плагин зарегистрирован: {name}")
        console.print(f"[dim]🔌 Плагин загружен: {name}[/dim]")
    
    def unregister_plugin(self, name: str) -> None:
        """Удаляет плагин.
        
        Args:
            name: Имя плагина
        """
        if name in self.plugins:
            del self.plugins[name]
            logger.info(f"Плагин удален: {name}")
    
    def get_plugin(self, name: str) -> Optional[BasePlugin]:
        """Получает плагин по имени.
        
        Args:
            name: Имя плагина
            
        Returns:
            BasePlugin: Плагин или None
        """
        return self.plugins.get(name)
    
    def list_plugins(self) -> List[Dict[str, Any]]:
        """Возвращает список всех плагинов.
        
        Returns:
            List[Dict]: Список информации о плагинах
        """
        return [
            {
                "name": name,
                "class": plugin.__class__.__name__,
                "version": plugin.get_version(),
                "hooks": self._get_plugin_hooks(plugin)
            }
            for name, plugin in self.plugins.items()
        ]
    
    def execute_hook(self, hook_name: str, context: Dict[str, Any]) -> None:
        """Выполняет хук во всех плагинах.
        
        Args:
            hook_name: Имя хука
            context: Контекст выполнения
        """
        for plugin in self.plugins.values():
            try:
                if hasattr(plugin, hook_name):
                    method = getattr(plugin, hook_name)
                    if callable(method):
                        method(context)
            except Exception as e:
                logger.error(f"Ошибка выполнения хука {hook_name} в плагине {plugin.name}: {e}")
                plugin.on_error(e, context)
    
    def execute_hook_with_result(self, hook_name: str, context: Dict[str, Any]) -> List[bool]:
        """Выполняет хук с возвратом результатов.
        
        Args:
            hook_name: Имя хука
            context: Контекст выполнения
            
        Returns:
            List[bool]: Список результатов выполнения
        """
        results = []
        for plugin in self.plugins.values():
            try:
                if hasattr(plugin, hook_name):
                    method = getattr(plugin, hook_name)
                    if callable(method):
                        result = method(context)
                        results.append(result if isinstance(result, bool) else True)
                    else:
                        results.append(True)
                else:
                    results.append(True)
            except Exception as e:
                logger.error(f"Ошибка выполнения хука {hook_name} в плагине {plugin.name}: {e}")
                plugin.on_error(e, context)
                results.append(False)
        return results
    
    def _load_builtin_plugins(self) -> None:
        """Загружает встроенные плагины."""
        try:
            # Загружаем плагины из текущего пакета
            from .telemetry import TelemetryPlugin
            from .validation import ValidationPlugin
            from .optimization import OptimizationPlugin
            
            # Регистрируем встроенные плагины
            self.register_plugin("telemetry", TelemetryPlugin(self.config))
            self.register_plugin("validation", ValidationPlugin(self.config))
            self.register_plugin("optimization", OptimizationPlugin(self.config))
            
        except ImportError as e:
            logger.warning(f"Не удалось загрузить встроенные плагины: {e}")
    
    def _get_plugin_hooks(self, plugin: BasePlugin) -> List[str]:
        """Получает список хуков плагина.
        
        Args:
            plugin: Плагин
            
        Returns:
            List[str]: Список хуков
        """
        hooks = []
        for method_name in dir(plugin):
            if method_name.startswith('on_'):
                hooks.append(method_name)
        return hooks
    
    def load_external_plugins(self) -> None:
        """Загружает внешние плагины из директории plugins."""
        if not self.plugins_dir.exists():
            return
        
        for plugin_file in self.plugins_dir.glob("*.py"):
            if plugin_file.name.startswith("_"):
                continue
            
            try:
                # Динамически импортируем модуль
                spec = importlib.util.spec_from_file_location(
                    plugin_file.stem, 
                    plugin_file
                )
                module = importlib.util.module_from_spec(spec)
                spec.loader.exec_module(module)
                
                # Ищем классы плагинов
                for name, obj in inspect.getmembers(module):
                    if (inspect.isclass(obj) and 
                        issubclass(obj, BasePlugin) and 
                        obj != BasePlugin):
                        
                        plugin = obj(self.config)
                        self.register_plugin(plugin.get_name(), plugin)
                        
            except Exception as e:
                logger.error(f"Ошибка загрузки плагина {plugin_file}: {e}")


 