"""Модуль семантических релизов для GLT CLI."""

import logging
import re
import subprocess
from datetime import datetime
from pathlib import Path
from typing import Dict, Any, List, Optional, Tuple

from packaging import version
from rich.console import Console
from rich.table import Table

console = Console()
logger = logging.getLogger(__name__)


class SemanticRelease:
    """Семантические релизы для GLT CLI."""
    
    def __init__(self, config: Any):
        """Инициализирует семантические релизы.
        
        Args:
            config: GLT конфигурация
        """
        self.config = config
        self.repo_path = Path.cwd()
        self.current_version = config.version
        
        # Настройки семантических релизов
        self.should_analyze_commits = getattr(config, 'semantic_analyze_commits', True)
        self.auto_release = getattr(config, 'semantic_auto_release', False)
        self.release_branch = getattr(config, 'semantic_release_branch', 'main')
        
        # Типы коммитов для анализа
        self.commit_types = {
            'major': ['BREAKING CHANGE', 'major'],
            'minor': ['feat', 'feature', 'minor'],
            'patch': ['fix', 'bugfix', 'patch', 'docs', 'style', 'refactor', 'test', 'chore']
        }
    
    def analyze_commits(self, since: Optional[str] = None) -> str:
        """Анализирует коммиты для определения версии.
        
        Args:
            since: Начиная с какого коммита анализировать
            
        Returns:
            str: Новая версия
        """
        try:
            console.print("[dim]🔍 Анализ коммитов...[/dim]")
            
            # Получаем коммиты
            commits = self._get_commits(since)
            if not commits:
                console.print("[dim]📝 Нет новых коммитов для анализа[/dim]")
                return self.current_version
            
            # Анализируем типы коммитов
            commit_types = self._analyze_commit_types(commits)
            
            # Определяем тип релиза
            release_type = self._determine_release_type(commit_types)
            
            # Генерируем новую версию
            new_version = self._generate_new_version(release_type)
            
            console.print(f"[green]📈 Определена версия: {new_version} ({release_type})[/green]")
            return new_version
            
        except Exception as e:
            logger.error(f"Ошибка анализа коммитов: {e}")
            console.print(f"[red]❌ Ошибка анализа коммитов: {e}[/red]")
            return self.current_version
    
    def create_release(self, version: str, dry_run: bool = False) -> bool:
        """Создает семантический релиз.
        
        Args:
            version: Версия для релиза
            dry_run: Тестовый запуск без создания релиза
            
        Returns:
            bool: True если релиз создан успешно
        """
        try:
            if dry_run:
                console.print(f"[dim]🧪 Тестовый релиз версии {version}[/dim]")
            else:
                console.print(f"[blue]🚀 Создание релиза версии {version}...[/blue]")
            
            # Проверяем, что мы на правильной ветке
            if not self._check_branch():
                console.print(f"[red]❌ Не на ветке {self.release_branch}[/red]")
                return False
            
            # Проверяем, что нет незакоммиченных изменений
            if not self._check_clean_working_dir():
                console.print("[red]❌ Есть незакоммиченные изменения[/red]")
                return False
            
            # Обновляем версию в конфигурации
            if not dry_run:
                self._update_version_in_config(version)
            
            # Создаем тег
            if not dry_run:
                self._create_git_tag(version)
            
            # Генерируем changelog
            changelog = self._generate_changelog(version)
            
            # Создаем релиз на GitHub (если настроено)
            if not dry_run and self._is_github_repo():
                self._create_github_release(version, changelog)
            
            if not dry_run:
                console.print(f"[green]✅ Релиз {version} создан успешно![/green]")
            else:
                console.print(f"[green]✅ Тестовый релиз {version} готов![/green]")
            
            return True
            
        except Exception as e:
            logger.error(f"Ошибка создания релиза: {e}")
            console.print(f"[red]❌ Ошибка создания релиза: {e}[/red]")
            return False
    
    def _get_commits(self, since: Optional[str] = None) -> List[Dict[str, str]]:
        """Получает список коммитов.
        
        Args:
            since: Начиная с какого коммита
            
        Returns:
            List[Dict]: Список коммитов
        """
        try:
            cmd = ["git", "log", "--pretty=format:%H|%s|%b", "--no-merges"]
            
            if since:
                cmd.extend([f"{since}..HEAD"])
            
            result = subprocess.run(cmd, capture_output=True, text=True, cwd=self.repo_path)
            
            if result.returncode != 0:
                return []
            
            commits = []
            for line in result.stdout.strip().split('\n'):
                if line:
                    parts = line.split('|', 2)
                    if len(parts) >= 2:
                        commits.append({
                            'hash': parts[0],
                            'subject': parts[1],
                            'body': parts[2] if len(parts) > 2 else ''
                        })
            
            return commits
            
        except Exception as e:
            logger.error(f"Ошибка получения коммитов: {e}")
            return []
    
    def _analyze_commit_types(self, commits: List[Dict[str, str]]) -> Dict[str, int]:
        """Анализирует типы коммитов.
        
        Args:
            commits: Список коммитов
            
        Returns:
            Dict: Статистика типов коммитов
        """
        commit_types = {'major': 0, 'minor': 0, 'patch': 0}
        
        for commit in commits:
            subject = commit['subject'].lower()
            body = commit['body'].lower()
            
            # Проверяем на breaking changes
            if any(keyword in subject or keyword in body for keyword in self.commit_types['major']):
                commit_types['major'] += 1
            elif any(keyword in subject for keyword in self.commit_types['minor']):
                commit_types['minor'] += 1
            else:
                commit_types['patch'] += 1
        
        return commit_types
    
    def _determine_release_type(self, commit_types: Dict[str, int]) -> str:
        """Определяет тип релиза.
        
        Args:
            commit_types: Статистика типов коммитов
            
        Returns:
            str: Тип релиза (major/minor/patch)
        """
        if commit_types['major'] > 0:
            return 'major'
        elif commit_types['minor'] > 0:
            return 'minor'
        else:
            return 'patch'
    
    def _generate_new_version(self, release_type: str) -> str:
        """Генерирует новую версию.
        
        Args:
            release_type: Тип релиза
            
        Returns:
            str: Новая версия
        """
        current = version.parse(self.current_version)
        
        if release_type == 'major':
            return f"{current.major + 1}.0.0"
        elif release_type == 'minor':
            return f"{current.major}.{current.minor + 1}.0"
        else:  # patch
            return f"{current.major}.{current.minor}.{current.micro + 1}"
    
    def _check_branch(self) -> bool:
        """Проверяет, что мы на правильной ветке."""
        try:
            result = subprocess.run(
                ["git", "branch", "--show-current"],
                capture_output=True,
                text=True,
                cwd=self.repo_path
            )
            
            current_branch = result.stdout.strip()
            return current_branch == self.release_branch
            
        except Exception:
            return False
    
    def _check_clean_working_dir(self) -> bool:
        """Проверяет, что рабочая директория чистая."""
        try:
            result = subprocess.run(
                ["git", "status", "--porcelain"],
                capture_output=True,
                text=True,
                cwd=self.repo_path
            )
            
            return not result.stdout.strip()
            
        except Exception:
            return False
    
    def _update_version_in_config(self, new_version: str) -> None:
        """Обновляет версию в конфигурации."""
        try:
            # Обновляем .glt файл
            glt_file = self.repo_path / ".glt"
            if glt_file.exists():
                content = glt_file.read_text()
                content = re.sub(
                    r'version:\s*[\d.]+',
                    f'version: {new_version}',
                    content
                )
                glt_file.write_text(content)
                
                # Коммитим изменения
                subprocess.run(["git", "add", ".glt"], cwd=self.repo_path)
                subprocess.run(
                    ["git", "commit", "-m", f"chore: bump version to {new_version}"],
                    cwd=self.repo_path
                )
                
        except Exception as e:
            logger.error(f"Ошибка обновления версии: {e}")
    
    def _create_git_tag(self, version: str) -> None:
        """Создает Git тег."""
        try:
            subprocess.run(
                ["git", "tag", f"v{version}"],
                cwd=self.repo_path
            )
            subprocess.run(
                ["git", "push", "origin", f"v{version}"],
                cwd=self.repo_path
            )
            
        except Exception as e:
            logger.error(f"Ошибка создания тега: {e}")
    
    def _generate_changelog(self, version: str) -> str:
        """Генерирует changelog."""
        try:
            # Получаем коммиты с последнего тега
            cmd = ["git", "log", "--pretty=format:- %s", "--no-merges"]
            
            # Находим последний тег
            result = subprocess.run(
                ["git", "describe", "--tags", "--abbrev=0"],
                capture_output=True,
                text=True,
                cwd=self.repo_path
            )
            
            if result.returncode == 0:
                last_tag = result.stdout.strip()
                cmd.extend([f"{last_tag}..HEAD"])
            
            result = subprocess.run(cmd, capture_output=True, text=True, cwd=self.repo_path)
            
            if result.returncode == 0:
                commits = result.stdout.strip().split('\n')
                commits = [commit for commit in commits if commit.strip()]
                
                changelog = f"# GLT CLI {version}\n\n"
                changelog += f"Релиз от {datetime.now().strftime('%Y-%m-%d')}\n\n"
                changelog += "## Изменения\n\n"
                
                for commit in commits:
                    changelog += f"{commit}\n"
                
                return changelog
            else:
                return f"# GLT CLI {version}\n\nРелиз от {datetime.now().strftime('%Y-%m-%d')}"
                
        except Exception as e:
            logger.error(f"Ошибка генерации changelog: {e}")
            return f"# GLT CLI {version}\n\nРелиз от {datetime.now().strftime('%Y-%m-%d')}"
    
    def _is_github_repo(self) -> bool:
        """Проверяет, является ли репозиторий GitHub."""
        try:
            result = subprocess.run(
                ["git", "remote", "get-url", "origin"],
                capture_output=True,
                text=True,
                cwd=self.repo_path
            )
            
            return "github.com" in result.stdout
            
        except Exception:
            return False
    
    def _create_github_release(self, version: str, changelog: str) -> None:
        """Создает релиз на GitHub."""
        try:
            # Используем GitHub CLI если доступен
            result = subprocess.run(
                ["gh", "--version"],
                capture_output=True,
                text=True
            )
            
            if result.returncode == 0:
                # Создаем релиз через GitHub CLI
                subprocess.run([
                    "gh", "release", "create",
                    f"v{version}",
                    "--title", f"GLT CLI {version}",
                    "--notes", changelog
                ], cwd=self.repo_path)
            else:
                console.print("[dim]💡 GitHub CLI не найден, релиз создан только локально[/dim]")
                
        except Exception as e:
            logger.error(f"Ошибка создания GitHub релиза: {e}")
    
    def show_release_info(self) -> None:
        """Показывает информацию о релизах."""
        try:
            # Получаем последние теги
            result = subprocess.run(
                ["git", "tag", "--sort=-version:refname", "-l", "v*"],
                capture_output=True,
                text=True,
                cwd=self.repo_path
            )
            
            if result.returncode == 0:
                tags = result.stdout.strip().split('\n')[:5]  # Последние 5 тегов
                
                table = Table(title="Последние релизы")
                table.add_column("Версия", style="cyan")
                table.add_column("Дата", style="magenta")
                table.add_column("Коммит", style="green")
                
                for tag in tags:
                    if tag:
                        # Получаем информацию о теге
                        commit_result = subprocess.run(
                            ["git", "rev-list", "-n", "1", tag],
                            capture_output=True,
                            text=True,
                            cwd=self.repo_path
                        )
                        
                        date_result = subprocess.run(
                            ["git", "log", "-1", "--format=%cd", "--date=short", tag],
                            capture_output=True,
                            text=True,
                            cwd=self.repo_path
                        )
                        
                        commit_hash = commit_result.stdout.strip()[:8] if commit_result.stdout else "N/A"
                        date = date_result.stdout.strip() if date_result.stdout else "N/A"
                        
                        table.add_row(tag, date, commit_hash)
                
                console.print(table)
            else:
                console.print("[dim]📝 Нет релизов[/dim]")
                
        except Exception as e:
            logger.error(f"Ошибка получения информации о релизах: {e}")
            console.print(f"[red]❌ Ошибка: {e}[/red]")


class ReleaseBot:
    """Бот для автоматических релизов."""
    
    def __init__(self, config: Any):
        """Инициализирует бота релизов.
        
        Args:
            config: GLT конфигурация
        """
        self.config = config
        self.semantic_release = SemanticRelease(config)
    
    def auto_release(self) -> bool:
        """Выполняет автоматический релиз."""
        try:
            if not self.config.repo.semantic_release:
                console.print("[dim]📝 Семантические релизы отключены[/dim]")
                return False
            
            # Анализируем коммиты
            new_version = self.semantic_release.analyze_commits()
            
            if new_version == self.config.version:
                console.print("[dim]📝 Нет изменений для релиза[/dim]")
                return False
            
            # Создаем релиз
            return self.semantic_release.create_release(new_version)
            
        except Exception as e:
            logger.error(f"Ошибка автоматического релиза: {e}")
            return False
    
    def should_auto_release(self) -> bool:
        """Проверяет, нужно ли автоматическое создание релиза."""
        return (
            self.config.repo.semantic_release and
            getattr(self.config, 'semantic_auto_release', False)
        ) 