"""Рендереры для генерации файлов проекта."""

from typing import List

from glt_cli.models import GLTConfig


class BaseRenderer:
    """Базовый класс для рендереров."""
    
    def __init__(self, config: GLTConfig):
        """Инициализирует рендерер.
        
        Args:
            config: GLT-конфигурация
        """
        self.config = config


class PythonRenderer(BaseRenderer):
    """Рендерер для Python файлов."""
    
    def render_pyproject(self) -> str:
        """Генерирует pyproject.toml."""
        package_name = self.config.meta.name.replace(".", "-")
        
        deps = "\n".join([f'    "{dep}",' for dep in self.config.stack.deps])
        dev_deps = "\n".join([f'    "{dep}",' for dep in self.config.stack.dev_deps])
        
        return f'''[build-system]
requires = ["hatchling"]
build-backend = "hatchling.build"

[project]
name = "{package_name}"
version = "0.1.0"
description = "{self.config.meta.name} - GLT модуль"
readme = "README.md"
license = {{text = "MIT"}}
authors = [
    {{name = "{self.config.meta.owner}", email = "team@example.com"}}
]
classifiers = [
    "Development Status :: 3 - Alpha",
    "Intended Audience :: Developers",
    "License :: OSI Approved :: MIT License",
    "Programming Language :: Python :: 3",
    "Programming Language :: Python :: 3.12",
]
requires-python = ">=3.12"
dependencies = [
{deps}
]

[project.optional-dependencies]
dev = [
{dev_deps}
]

[project.scripts]
{package_name.replace("-", "_")} = "{self.config.meta.name.replace(".", "_")}.main:main"

[tool.hatch.build.targets.wheel]
packages = ["{self.config.meta.name.replace(".", "_")}"]

[tool.ruff]
target-version = "py312"
line-length = 88
select = ["E", "W", "F", "I", "B", "C4", "UP"]

[tool.mypy]
python_version = "3.12"
warn_return_any = true
disallow_untyped_defs = true

[tool.pytest.ini_options]
testpaths = ["tests"]
python_files = ["test_*.py"]
addopts = ["--cov={self.config.meta.name.replace(".", "_")}", "--cov-report=term-missing"]
'''
    
    def render_init(self) -> str:
        """Генерирует __init__.py."""
        return f'''"""{self.config.meta.name} - GLT модуль."""

__version__ = "0.1.0"
__author__ = "{self.config.meta.owner}"
__email__ = "team@example.com"
'''
    
    def render_main(self) -> str:
        """Генерирует main.py."""
        return f'''"""Основной модуль {self.config.meta.name}."""

import sys
from pathlib import Path

import typer
from rich.console import Console

console = Console()

app = typer.Typer(
    name="{self.config.meta.name.split('.')[-1]}",
    help="{self.config.meta.name} - GLT модуль",
    add_completion=False,
)


@app.command()
def main():
    """Основная команда."""
    console.print(f"[bold blue]🚀 {self.config.meta.name}[/bold blue]")
    console.print(f"📋 Версия: {self.config.version}")
    console.print(f"🌊 Волна: {self.config.meta.wave}")


if __name__ == "__main__":
    app()
'''
    
    def render_models(self) -> str:
        """Генерирует models.py."""
        return '''"""Модели данных."""

from typing import Optional
from pydantic import BaseModel, Field


class Config(BaseModel):
    """Конфигурация модуля."""
    
    name: str = Field(..., description="Имя модуля")
    version: str = Field(..., description="Версия")
    description: Optional[str] = Field(None, description="Описание")
'''
    
    def render_utils(self) -> str:
        """Генерирует utils.py."""
        return '''"""Утилиты."""

import logging
from pathlib import Path

logger = logging.getLogger(__name__)


def setup_logging(verbose: bool = False) -> None:
    """Настраивает логирование."""
    level = logging.DEBUG if verbose else logging.INFO
    logging.basicConfig(
        level=level,
        format="%(asctime)s - %(name)s - %(levelname)s - %(message)s"
    )
'''


class DockerRenderer(BaseRenderer):
    """Рендерер для Docker файлов."""
    
    def render_dockerfile(self) -> str:
        """Генерирует Dockerfile."""
        runtime = self.config.stack.runtime
        package_name = self.config.meta.name.replace(".", "-")
        
        return f'''# Multi-stage build для {self.config.meta.name}
FROM {runtime} as base

# Устанавливаем зависимости
COPY pyproject.toml .
RUN pip install --no-cache-dir -e .

# Stage для разработки
FROM base as dev
RUN pip install --no-cache-dir pytest mypy ruff black

# Stage для тестирования
FROM dev as test
COPY . .
RUN pytest

# Stage для продакшена
FROM base as prod
COPY src/ /app/src/
WORKDIR /app

CMD ["python", "-m", "{self.config.meta.name.replace('.', '_')}"]
'''
    
    def render_compose(self) -> str:
        """Генерирует docker-compose.yml."""
        package_name = self.config.meta.name.replace(".", "-")
        
        return f'''version: '3.8'

services:
  {package_name}:
    build:
      context: .
      dockerfile: docker/Dockerfile
      target: prod
    ports:
      - "8000:8000"
    environment:
      - PYTHONPATH=/app
    volumes:
      - ./data:/app/data
'''


class CIRenderer(BaseRenderer):
    """Рендерер для CI конфигурации."""
    
    def render_workflow(self) -> str:
        """Генерирует GitHub Actions workflow."""
        package_name = self.config.meta.name.replace(".", "-")
        
        return f'''name: CI

on:
  push:
    branches: [ main, develop ]
  pull_request:
    branches: [ main ]

jobs:
  test:
    runs-on: ubuntu-latest
    strategy:
      matrix:
        python-version: ["3.12"]
    
    steps:
    - uses: actions/checkout@v4
    
    - name: Set up Python ${{{{ matrix.python-version }}}}
      uses: actions/setup-python@v4
      with:
        python-version: ${{{{ matrix.python-version }}}}
    
    - name: Install dependencies
      run: |
        python -m pip install --upgrade pip
        pip install -e ".[dev]"
    
    - name: Lint with ruff
      run: |
        ruff check .
        ruff format --check .
    
    - name: Type check with mypy
      run: |
        mypy src/
    
    - name: Test with pytest
      run: |
        pytest --cov={self.config.meta.name.replace('.', '_')} --cov-report=xml
    
    - name: Upload coverage to Codecov
      uses: codecov/codecov-action@v3
      with:
        file: ./coverage.xml
        fail_ci_if_error: true

  build:
    needs: test
    runs-on: ubuntu-latest
    if: github.ref == 'refs/heads/main'
    
    steps:
    - uses: actions/checkout@v4
    
    - name: Set up Python
      uses: actions/setup-python@v4
      with:
        python-version: "3.12"
    
    - name: Build package
      run: |
        pip install build
        python -m build
    
    - name: Upload artifacts
      uses: actions/upload-artifact@v3
      with:
        name: dist
        path: dist/
'''


class EnvRenderer(BaseRenderer):
    """Рендерер для переменных окружения."""
    
    def render_direnv(self) -> str:
        """Генерирует .envrc файл для direnv."""
        package_name = self.config.meta.name.replace(".", "_")
        
        return f"""# .envrc для {self.config.meta.name}
export PYTHONPATH="$PWD/src:$PYTHONPATH"
export GLT_VERSION="{self.config.version}"
export GLT_MODULE="{self.config.meta.name}"
export GLT_WAVE="{self.config.meta.wave}"

# Добавляем зависимости в PATH если нужно
if [ -d "venv" ]; then
    export PATH="$PWD/venv/bin:$PATH"
fi

# Уведомляем о загрузке
echo "🌊 GLT модуль {self.config.meta.name} (волна {self.config.meta.wave}) загружен"
"""
    
    def render_env_example(self) -> str:
        """Генерирует .env.example файл."""
        return f"""# Переменные окружения для {self.config.meta.name}
# Скопируйте этот файл в .env и заполните значения

# Основные настройки
GLT_VERSION={self.config.version}
GLT_MODULE={self.config.meta.name}
GLT_WAVE={self.config.meta.wave}

# Секреты (заполните своими значениями)
# API_KEY=your_api_key_here
# DATABASE_URL=your_database_url_here

# Настройки разработки
DEBUG=true
LOG_LEVEL=INFO
"""


class ReadmeRenderer(BaseRenderer):
    """Рендерер для README файлов."""
    
    def render_readme(self) -> str:
        """Генерирует README.md."""
        package_name = self.config.meta.name.replace(".", "-")
        
        return f'''# {self.config.meta.name}

{self.config.meta.name} - GLT модуль, волна {self.config.meta.wave}.

## 📋 Описание

Этот модуль был сгенерирован с помощью GLT CLI.

## 🚀 Установка

```bash
pip install {package_name}
```

## 📦 Разработка

```bash
# Клонирование
git clone <repository-url>
cd {package_name}

# Установка зависимостей
pip install -e ".[dev]"

# Запуск тестов
pytest

# Линтинг
ruff check .
ruff format .

# Типизация
mypy src/
```

## 🐳 Docker

```bash
# Сборка образа
docker build -f docker/Dockerfile -t {package_name} .

# Запуск
docker run {package_name}
```

## 📊 CI/CD

Проект использует GitHub Actions для автоматизации:
- Линтинг и форматирование кода
- Проверка типов
- Запуск тестов
- Сборка пакета

## 📄 Лицензия

MIT License

## 👥 Авторы

{self.config.meta.owner}

---
*Сгенерировано с помощью GLT CLI*
''' 