"""Модуль безопасности GLT CLI."""

import logging
import subprocess
import tempfile
from pathlib import Path
from typing import Dict, Any, Optional

from rich.console import Console

from glt_cli.models import GLTConfig

console = Console()
logger = logging.getLogger(__name__)


class SOPSManager:
    """Менеджер для работы с SOPS шифрованием."""
    
    def __init__(self, config: GLTConfig):
        """Инициализирует SOPS менеджер.
        
        Args:
            config: GLT конфигурация
        """
        self.config = config
        self.sops_config = config.security.sops if config.security else None
    
    def is_sops_available(self) -> bool:
        """Проверяет доступность SOPS."""
        try:
            result = subprocess.run(
                ["sops", "--version"], 
                capture_output=True, 
                text=True, 
                timeout=5
            )
            return result.returncode == 0
        except (subprocess.TimeoutExpired, FileNotFoundError):
            return False
    
    def encrypt_file(self, file_path: Path, output_path: Optional[Path] = None) -> bool:
        """Шифрует файл с помощью SOPS.
        
        Args:
            file_path: Путь к файлу для шифрования
            output_path: Путь для сохранения зашифрованного файла
            
        Returns:
            bool: True если шифрование успешно
        """
        if not self.is_sops_available():
            console.print("[yellow]⚠️  SOPS не установлен или недоступен[/yellow]")
            return False
        
        if not self.sops_config:
            console.print("[yellow]⚠️  SOPS конфигурация не найдена[/yellow]")
            return False
        
        if output_path is None:
            output_path = file_path.with_suffix(file_path.suffix + ".enc")
        
        try:
            # Получаем KMS ключ из конфигурации
            kms_key = self.sops_config.get("kms")
            if not kms_key:
                console.print("[red]❌ KMS ключ не указан в конфигурации SOPS[/red]")
                return False
            
            # Команда шифрования
            cmd = [
                "sops",
                "--encrypt",
                "--kms", kms_key,
                "--output", str(output_path),
                str(file_path)
            ]
            
            logger.info(f"Шифруем файл {file_path} -> {output_path}")
            
            result = subprocess.run(
                cmd,
                capture_output=True,
                text=True,
                timeout=30
            )
            
            if result.returncode == 0:
                console.print(f"[green]✅ Файл зашифрован: {output_path}[/green]")
                return True
            else:
                console.print(f"[red]❌ Ошибка шифрования: {result.stderr}[/red]")
                return False
                
        except subprocess.TimeoutExpired:
            console.print("[red]❌ Таймаут при шифровании[/red]")
            return False
        except Exception as e:
            console.print(f"[red]❌ Ошибка шифрования: {e}[/red]")
            return False
    
    def decrypt_file(self, file_path: Path, output_path: Optional[Path] = None) -> bool:
        """Расшифровывает файл с помощью SOPS.
        
        Args:
            file_path: Путь к зашифрованному файлу
            output_path: Путь для сохранения расшифрованного файла
            
        Returns:
            bool: True если расшифрование успешно
        """
        if not self.is_sops_available():
            console.print("[yellow]⚠️  SOPS не установлен или недоступен[/yellow]")
            return False
        
        if output_path is None:
            output_path = file_path.with_suffix("").with_suffix(file_path.suffix.replace(".enc", ""))
        
        try:
            cmd = [
                "sops",
                "--decrypt",
                "--output", str(output_path),
                str(file_path)
            ]
            
            logger.info(f"Расшифровываем файл {file_path} -> {output_path}")
            
            result = subprocess.run(
                cmd,
                capture_output=True,
                text=True,
                timeout=30
            )
            
            if result.returncode == 0:
                console.print(f"[green]✅ Файл расшифрован: {output_path}[/green]")
                return True
            else:
                console.print(f"[red]❌ Ошибка расшифрования: {result.stderr}[/red]")
                return False
                
        except subprocess.TimeoutExpired:
            console.print("[red]❌ Таймаут при расшифровании[/red]")
            return False
        except Exception as e:
            console.print(f"[red]❌ Ошибка расшифрования: {e}[/red]")
            return False
    
    def encrypt_secrets(self, secrets: Dict[str, Any]) -> bool:
        """Шифрует секреты и сохраняет в файл.
        
        Args:
            secrets: Словарь с секретами
            
        Returns:
            bool: True если шифрование успешно
        """
        if not secrets:
            console.print("[yellow]⚠️  Нет секретов для шифрования[/yellow]")
            return True
        
        try:
            # Создаем временный файл с секретами
            with tempfile.NamedTemporaryFile(mode='w', suffix='.yaml', delete=False) as f:
                import yaml
                yaml.dump(secrets, f, default_flow_style=False)
                temp_file = Path(f.name)
            
            # Шифруем файл
            encrypted_file = Path("secrets.enc.yaml")
            success = self.encrypt_file(temp_file, encrypted_file)
            
            # Удаляем временный файл
            temp_file.unlink()
            
            return success
            
        except Exception as e:
            console.print(f"[red]❌ Ошибка шифрования секретов: {e}[/red]")
            return False
    
    def create_sops_config(self, output_path: Path) -> bool:
        """Создает конфигурационный файл SOPS.
        
        Args:
            output_path: Путь для сохранения конфигурации
            
        Returns:
            bool: True если создание успешно
        """
        if not self.sops_config:
            console.print("[yellow]⚠️  SOPS конфигурация не найдена[/yellow]")
            return False
        
        try:
            config_content = f"""# SOPS конфигурация для {self.config.meta.name}
creation_rules:
  - path_regex: \\.enc\\.(yaml|yml|json)$
    kms: "{self.sops_config.get('kms', '')}"
    age: >-
      age1ql3z7hjy54pw3hyww5ayyfg7zqgvc7w3j2elw8zmrj2kg5sfn9aqmcac8p
"""
            
            output_path.write_text(config_content, encoding='utf-8')
            console.print(f"[green]✅ SOPS конфигурация создана: {output_path}[/green]")
            return True
            
        except Exception as e:
            console.print(f"[red]❌ Ошибка создания SOPS конфигурации: {e}[/red]")
            return False


class SecurityManager:
    """Общий менеджер безопасности."""
    
    def __init__(self, config: GLTConfig):
        """Инициализирует менеджер безопасности.
        
        Args:
            config: GLT конфигурация
        """
        self.config = config
        self.sops = SOPSManager(config) if config.security else None
    
    def setup_security(self, project_dir: Path) -> bool:
        """Настраивает безопасность проекта.
        
        Args:
            project_dir: Директория проекта
            
        Returns:
            bool: True если настройка успешна
        """
        if not self.config.security:
            console.print("[yellow]⚠️  Конфигурация безопасности не найдена[/yellow]")
            return True
        
        success = True
        
        # Создаем .sops.yaml если есть SOPS конфигурация
        if self.sops and self.config.security.sops:
            sops_config_path = project_dir / ".sops.yaml"
            if not self.sops.create_sops_config(sops_config_path):
                success = False
        
        # Создаем pre-commit конфигурацию
        if self.config.security.pre_commit:
            pre_commit_path = project_dir / ".pre-commit-config.yaml"
            if not self._create_pre_commit_config(pre_commit_path):
                success = False
        
        return success
    
    def _create_pre_commit_config(self, output_path: Path) -> bool:
        """Создает конфигурацию pre-commit.
        
        Args:
            output_path: Путь для сохранения конфигурации
            
        Returns:
            bool: True если создание успешно
        """
        try:
            hooks = []
            
            for hook in self.config.security.pre_commit:
                if hook == "detect-secrets":
                    hooks.append({
                        "repo": "https://github.com/Yelp/detect-secrets",
                        "rev": "v1.4.0",
                        "hooks": [{"id": "detect-secrets"}]
                    })
                elif hook == "ruff":
                    hooks.append({
                        "repo": "https://github.com/astral-sh/ruff-pre-commit",
                        "rev": "v0.1.6",
                        "hooks": [{"id": "ruff"}, {"id": "ruff-format"}]
                    })
                elif hook == "mypy":
                    hooks.append({
                        "repo": "https://github.com/pre-commit/mirrors-mypy",
                        "rev": "v1.7.1",
                        "hooks": [{"id": "mypy"}]
                    })
            
            config_content = {
                "repos": hooks
            }
            
            import yaml
            with open(output_path, 'w', encoding='utf-8') as f:
                yaml.dump(config_content, f, default_flow_style=False)
            
            console.print(f"[green]✅ Pre-commit конфигурация создана: {output_path}[/green]")
            return True
            
        except Exception as e:
            console.print(f"[red]❌ Ошибка создания pre-commit конфигурации: {e}[/red]")
            return False 