"""Модуль автообновления GLT CLI."""

import json
import logging
import subprocess
import sys
from pathlib import Path
from typing import Dict, Any, Optional, Tuple
from urllib.parse import urljoin

import requests
from packaging import version
from rich.console import Console
from rich.progress import Progress, SpinnerColumn, TextColumn

console = Console()
logger = logging.getLogger(__name__)


class AutoUpdater:
    """Автоматическое обновление GLT CLI."""
    
    def __init__(self, config: Any):
        """Инициализирует автообновление.
        
        Args:
            config: GLT конфигурация
        """
        self.config = config
        self.current_version = "0.6.0"  # Текущая версия CLI
        self.update_url = "https://api.github.com/repos/Quantum-Insight-Lab/glt-cli/releases/latest"
        self.download_url = "https://github.com/Quantum-Insight-Lab/glt-cli/releases/download"
        
        # Настройки обновления
        self.auto_check = getattr(config, 'auto_update_check', True)
        self.check_interval = getattr(config, 'update_check_interval', 86400)  # 24 часа
        
        # Файл для отслеживания последней проверки
        self.last_check_file = Path.home() / ".glt" / "last_update_check"
        self.last_check_file.parent.mkdir(parents=True, exist_ok=True)
    
    def check_for_updates(self, force: bool = False) -> Optional[str]:
        """Проверяет наличие обновлений.
        
        Args:
            force: Принудительная проверка
            
        Returns:
            str: Новая версия или None
        """
        if not force and not self._should_check():
            return None
        
        try:
            console.print("[dim]🔍 Проверка обновлений...[/dim]")
            
            # Получаем информацию о последнем релизе
            response = requests.get(self.update_url, timeout=10)
            response.raise_for_status()
            
            release_info = response.json()
            latest_version = release_info['tag_name'].lstrip('v')
            
            # Сравниваем версии
            if version.parse(latest_version) > version.parse(self.current_version):
                console.print(f"[yellow]🔄 Доступно обновление: {latest_version}[/yellow]")
                self._save_last_check()
                return latest_version
            else:
                console.print("[dim]✅ У вас последняя версия[/dim]")
                self._save_last_check()
                return None
                
        except requests.RequestException as e:
            logger.error(f"Ошибка проверки обновлений: {e}")
            console.print("[dim]⚠️  Не удалось проверить обновления[/dim]")
            return None
        except Exception as e:
            logger.error(f"Неожиданная ошибка при проверке обновлений: {e}")
            return None
    
    def update(self, target_version: Optional[str] = None) -> bool:
        """Обновляет CLI.
        
        Args:
            target_version: Целевая версия (если None, то последняя)
            
        Returns:
            bool: True если обновление успешно
        """
        try:
            if not target_version:
                target_version = self.check_for_updates(force=True)
                if not target_version:
                    console.print("[yellow]⚠️  Нет доступных обновлений[/yellow]")
                    return False
            
            console.print(f"[blue]🔄 Обновление до версии {target_version}...[/blue]")
            
            with Progress(
                SpinnerColumn(),
                TextColumn("[progress.description]{task.description}"),
                console=console,
            ) as progress:
                
                # Задача 1: Скачивание
                task1 = progress.add_task("Скачивание обновления...", total=None)
                
                # Определяем тип установки
                if self._is_pip_install():
                    success = self._update_via_pip(target_version)
                else:
                    success = self._update_via_github(target_version)
                
                progress.update(task1, completed=True)
                
                if success:
                    # Задача 2: Установка
                    task2 = progress.add_task("Установка обновления...", total=None)
                    progress.update(task2, completed=True)
                    
                    console.print(f"[green]✅ GLT CLI обновлен до версии {target_version}[/green]")
                    console.print("[dim]💡 Перезапустите терминал для применения изменений[/dim]")
                    return True
                else:
                    console.print("[red]❌ Ошибка обновления[/red]")
                    return False
                    
        except Exception as e:
            logger.error(f"Ошибка обновления: {e}")
            console.print(f"[red]❌ Ошибка обновления: {e}[/red]")
            return False
    
    def _should_check(self) -> bool:
        """Проверяет, нужно ли выполнить проверку обновлений."""
        if not self.auto_check:
            return False
        
        if not self.last_check_file.exists():
            return True
        
        try:
            last_check = self.last_check_file.read_text().strip()
            import time
            current_time = int(time.time())
            
            # Проверяем, прошло ли достаточно времени
            return (current_time - int(last_check)) > self.check_interval
            
        except Exception:
            return True
    
    def _save_last_check(self) -> None:
        """Сохраняет время последней проверки."""
        try:
            import time
            self.last_check_file.write_text(str(int(time.time())))
        except Exception as e:
            logger.error(f"Ошибка сохранения времени проверки: {e}")
    
    def _is_pip_install(self) -> bool:
        """Проверяет, установлен ли CLI через pip."""
        try:
            # Проверяем, есть ли файл setup.py или pyproject.toml в корне
            cli_root = Path(__file__).parent.parent
            return (cli_root / "setup.py").exists() or (cli_root / "pyproject.toml").exists()
        except Exception:
            return False
    
    def _update_via_pip(self, target_version: str) -> bool:
        """Обновляет через pip.
        
        Args:
            target_version: Целевая версия
            
        Returns:
            bool: True если успешно
        """
        try:
            cmd = [
                sys.executable, "-m", "pip", "install", 
                "--upgrade", f"glt-cli=={target_version}"
            ]
            
            result = subprocess.run(
                cmd,
                capture_output=True,
                text=True,
                timeout=60
            )
            
            if result.returncode == 0:
                return True
            else:
                logger.error(f"Ошибка pip обновления: {result.stderr}")
                return False
                
        except subprocess.TimeoutExpired:
            logger.error("Таймаут pip обновления")
            return False
        except Exception as e:
            logger.error(f"Ошибка pip обновления: {e}")
            return False
    
    def _update_via_github(self, target_version: str) -> bool:
        """Обновляет через GitHub релизы.
        
        Args:
            target_version: Целевая версия
            
        Returns:
            bool: True если успешно
        """
        try:
            # Определяем платформу
            import platform
            system = platform.system().lower()
            machine = platform.machine().lower()
            
            if system == "linux":
                if machine in ["x86_64", "amd64"]:
                    asset_name = f"glt-cli-{target_version}-linux-x86_64"
                elif machine == "aarch64":
                    asset_name = f"glt-cli-{target_version}-linux-aarch64"
                else:
                    console.print(f"[red]❌ Неподдерживаемая архитектура: {machine}[/red]")
                    return False
            elif system == "darwin":
                if machine == "x86_64":
                    asset_name = f"glt-cli-{target_version}-macos-x86_64"
                elif machine == "arm64":
                    asset_name = f"glt-cli-{target_version}-macos-arm64"
                else:
                    console.print(f"[red]❌ Неподдерживаемая архитектура: {machine}[/red]")
                    return False
            elif system == "windows":
                asset_name = f"glt-cli-{target_version}-windows-x86_64.exe"
            else:
                console.print(f"[red]❌ Неподдерживаемая ОС: {system}[/red]")
                return False
            
            # Скачиваем файл
            download_url = f"{self.download_url}/v{target_version}/{asset_name}"
            
            response = requests.get(download_url, stream=True, timeout=30)
            response.raise_for_status()
            
            # Сохраняем во временную директорию
            import tempfile
            with tempfile.NamedTemporaryFile(delete=False) as f:
                for chunk in response.iter_content(chunk_size=8192):
                    f.write(chunk)
                temp_file = f.name
            
            # Делаем исполняемым (для Unix)
            if system != "windows":
                import os
                os.chmod(temp_file, 0o755)
            
            # Заменяем текущий исполняемый файл
            current_exe = Path(sys.executable)
            backup_exe = current_exe.with_suffix(current_exe.suffix + ".backup")
            
            # Создаем резервную копию
            import shutil
            shutil.copy2(current_exe, backup_exe)
            
            # Заменяем файл
            shutil.move(temp_file, current_exe)
            
            return True
            
        except Exception as e:
            logger.error(f"Ошибка GitHub обновления: {e}")
            return False
    
    def get_update_info(self) -> Dict[str, Any]:
        """Получает информацию об обновлениях.
        
        Returns:
            Dict: Информация об обновлениях
        """
        try:
            response = requests.get(self.update_url, timeout=10)
            response.raise_for_status()
            
            release_info = response.json()
            latest_version = release_info['tag_name'].lstrip('v')
            
            return {
                "current_version": self.current_version,
                "latest_version": latest_version,
                "update_available": version.parse(latest_version) > version.parse(self.current_version),
                "release_notes": release_info.get('body', ''),
                "download_url": release_info.get('html_url', ''),
                "published_at": release_info.get('published_at', ''),
                "last_check": self._get_last_check_time()
            }
            
        except Exception as e:
            logger.error(f"Ошибка получения информации об обновлениях: {e}")
            return {
                "current_version": self.current_version,
                "error": str(e)
            }
    
    def _get_last_check_time(self) -> Optional[str]:
        """Получает время последней проверки."""
        try:
            if self.last_check_file.exists():
                timestamp = int(self.last_check_file.read_text().strip())
                import datetime
                return datetime.datetime.fromtimestamp(timestamp).isoformat()
        except Exception:
            pass
        return None


class UpdateChecker:
    """Проверка обновлений в фоновом режиме."""
    
    def __init__(self, config: Any):
        """Инициализирует проверку обновлений.
        
        Args:
            config: GLT конфигурация
        """
        self.config = config
        self.updater = AutoUpdater(config)
    
    def check_and_notify(self) -> None:
        """Проверяет обновления и уведомляет пользователя."""
        try:
            latest_version = self.updater.check_for_updates()
            if latest_version:
                console.print(f"[yellow]🔄 Доступно обновление GLT CLI: {latest_version}[/yellow]")
                console.print("[dim]💡 Запустите 'glt update' для обновления[/dim]")
        except Exception as e:
            logger.debug(f"Ошибка проверки обновлений: {e}")
    
    def should_auto_update(self) -> bool:
        """Проверяет, нужно ли автоматическое обновление."""
        return getattr(self.config, 'auto_update', False)
    
    def auto_update(self) -> bool:
        """Выполняет автоматическое обновление."""
        try:
            return self.updater.update()
        except Exception as e:
            logger.error(f"Ошибка автоматического обновления: {e}")
            return False 