"""Утилиты для работы с GLT CLI."""

import logging
from pathlib import Path
from typing import Dict, Any

import ruamel.yaml
from rich.console import Console

from glt_cli.models import GLTConfig

console = Console()
logger = logging.getLogger(__name__)


def load_glt_config(project_path: Path) -> GLTConfig:
    """Загружает GLT-конфигурацию из файла .glt.
    
    Args:
        project_path: Путь к проекту
        
    Returns:
        GLTConfig: Загруженная конфигурация
        
    Raises:
        FileNotFoundError: Если файл .glt не найден
        ValueError: Если конфигурация некорректна
    """
    glt_file = project_path / ".glt"
    
    if not glt_file.exists():
        # Ищем файл с расширением .glt
        glt_files = list(project_path.glob("*.glt"))
        if glt_files:
            glt_file = glt_files[0]
        else:
            raise FileNotFoundError(
                f"Файл .glt не найден в директории {project_path}"
            )
    
    logger.info(f"Загружаем конфигурацию из {glt_file}")
    
    try:
        # Загружаем YAML
        yaml = ruamel.yaml.YAML(typ='safe')
        with open(glt_file, 'r', encoding='utf-8') as f:
            data = yaml.load(f)
        
        if not data:
            raise ValueError("Файл .glt пуст или некорректен")
        
        # Преобразуем в GLTConfig
        config = GLTConfig(**data)
        
        logger.info(f"Конфигурация загружена: {config.meta.name}")
        return config
        
    except ruamel.yaml.YAMLError as e:
        raise ValueError(f"Ошибка парсинга YAML: {e}")
    except Exception as e:
        raise ValueError(f"Ошибка загрузки конфигурации: {e}")


def find_glt_projects(root_path: Path) -> list[Path]:
    """Находит все GLT-проекты в указанной директории.
    
    Args:
        root_path: Корневая директория для поиска
        
    Returns:
        list[Path]: Список путей к GLT-проектам
    """
    projects = []
    
    # Ищем файлы .glt
    for glt_file in root_path.rglob("*.glt"):
        project_dir = glt_file.parent
        if project_dir not in projects:
            projects.append(project_dir)
    
    # Ищем файлы .glt (без расширения)
    for glt_file in root_path.rglob(".glt"):
        project_dir = glt_file.parent
        if project_dir not in projects:
            projects.append(project_dir)
    
    return projects


def validate_project_structure(project_path: Path) -> bool:
    """Проверяет структуру GLT-проекта.
    
    Args:
        project_path: Путь к проекту
        
    Returns:
        bool: True если структура корректна
    """
    required_files = [".glt"]
    optional_dirs = ["templates", "src", "tests", "docs"]
    
    # Проверяем обязательные файлы
    for file_name in required_files:
        file_path = project_path / file_name
        if not file_path.exists():
            console.print(f"[yellow]⚠️  Отсутствует {file_name}[/yellow]")
            return False
    
    # Проверяем опциональные директории
    for dir_name in optional_dirs:
        dir_path = project_path / dir_name
        if dir_path.exists() and not dir_path.is_dir():
            console.print(f"[yellow]⚠️  {dir_name} должен быть директорией[/yellow]")
            return False
    
    return True


def get_project_info(project_path: Path) -> Dict[str, Any]:
    """Получает информацию о GLT-проекте.
    
    Args:
        project_path: Путь к проекту
        
    Returns:
        Dict[str, Any]: Информация о проекте
    """
    try:
        config = load_glt_config(project_path)
        
        return {
            "name": config.meta.name,
            "version": config.version,
            "wave": config.meta.wave,
            "owner": config.meta.owner,
            "visibility": config.meta.visibility,
            "runtime": config.stack.runtime,
            "dependencies_count": len(config.stack.deps),
            "dev_dependencies_count": len(config.stack.dev_deps),
            "has_repo": config.repo is not None,
            "has_docker": config.docker is not None,
            "has_ci": config.ci is not None,
            "plan_waves": len(config.plan),
        }
        
    except Exception as e:
        logger.error(f"Ошибка получения информации о проекте: {e}")
        return {"error": str(e)} 