"""Тесты для утилит."""

import tempfile
from pathlib import Path

import pytest
import ruamel.yaml

from models import GLTConfig, MetaInfo, Stack
import sys
import os
sys.path.insert(0, os.path.join(os.path.dirname(__file__), '..', 'glt_cli'))
from utils import load_glt_config, validate_project_structure


class TestLoadGLTConfig:
    """Тесты для загрузки GLT конфигурации."""
    
    def test_load_valid_config(self):
        """Тест загрузки валидной конфигурации."""
        config_data = {
            "version": "0.6",
            "meta": {
                "name": "test.module",
                "wave": "Æ3",
                "owner": "@test_user",
                "visibility": "private"
            },
            "stack": {
                "runtime": "python:3.12-slim",
                "deps": ["typer[all]>=0.9"],
                "dev_deps": ["pytest"]
            }
        }
        
        with tempfile.TemporaryDirectory() as temp_dir:
            temp_path = Path(temp_dir)
            glt_file = temp_path / ".glt"
            
            # Создаем .glt файл
            yaml = ruamel.yaml.YAML(typ='safe')
            with open(glt_file, 'w', encoding='utf-8') as f:
                yaml.dump(config_data, f)
            
            # Загружаем конфигурацию
            config = load_glt_config(temp_path)
            
            # Проверяем, что это объект с нужными атрибутами
            assert hasattr(config, 'version')
            assert hasattr(config, 'meta')
            assert hasattr(config, 'stack')
            assert config.version == "0.6"
            assert config.meta.name == "test.module"
            assert config.stack.runtime == "python:3.12-slim"
    
    def test_load_missing_file(self):
        """Тест загрузки при отсутствии файла."""
        with tempfile.TemporaryDirectory() as temp_dir:
            temp_path = Path(temp_dir)
            
            with pytest.raises(FileNotFoundError):
                load_glt_config(temp_path)
    
    def test_load_invalid_yaml(self):
        """Тест загрузки некорректного YAML."""
        with tempfile.TemporaryDirectory() as temp_dir:
            temp_path = Path(temp_dir)
            glt_file = temp_path / ".glt"
            
            # Создаем некорректный YAML
            with open(glt_file, 'w', encoding='utf-8') as f:
                f.write("invalid: yaml: content: [")
            
            with pytest.raises(ValueError, match="Ошибка парсинга YAML"):
                load_glt_config(temp_path)
    
    def test_load_empty_file(self):
        """Тест загрузки пустого файла."""
        with tempfile.TemporaryDirectory() as temp_dir:
            temp_path = Path(temp_dir)
            glt_file = temp_path / ".glt"
            
            # Создаем пустой файл
            glt_file.touch()
            
            with pytest.raises(ValueError, match="Файл .glt пуст или некорректен"):
                load_glt_config(temp_path)


class TestValidateProjectStructure:
    """Тесты для валидации структуры проекта."""
    
    def test_valid_structure(self):
        """Тест валидной структуры проекта."""
        with tempfile.TemporaryDirectory() as temp_dir:
            temp_path = Path(temp_dir)
            
            # Создаем .glt файл
            (temp_path / ".glt").touch()
            
            # Создаем опциональные директории
            (temp_path / "templates").mkdir()
            (temp_path / "src").mkdir()
            (temp_path / "tests").mkdir()
            
            assert validate_project_structure(temp_path) is True
    
    def test_missing_glt_file(self):
        """Тест отсутствия .glt файла."""
        with tempfile.TemporaryDirectory() as temp_dir:
            temp_path = Path(temp_dir)
            
            assert validate_project_structure(temp_path) is False
    
    def test_optional_dir_as_file(self):
        """Тест когда опциональная директория является файлом."""
        with tempfile.TemporaryDirectory() as temp_dir:
            temp_path = Path(temp_dir)
            
            # Создаем .glt файл
            (temp_path / ".glt").touch()
            
            # Создаем файл вместо директории
            (temp_path / "templates").touch()
            
            assert validate_project_structure(temp_path) is False 